@@ -23,12 +23,11 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- *      $Id: ftrec.c,v 1.4 2001/07/15 19:18:49 maf Exp $
+ *      $Id: ftrec.c,v 1.12 2003/02/13 02:38:42 maf Exp $
  */
 
-#if HAVE_CONFIG_H
- #include <config.h>
-#endif
+#include "ftconfig.h"
+#include "ftlib.h"
 
 #include <sys/time.h>
 #include <sys/types.h>
@@ -37,15 +36,14 @@
 #include <stddef.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <time.h>
+
 #if HAVE_STRINGS_H
  #include <strings.h>
 #endif
 #if HAVE_STRING_H
   #include <string.h>
 #endif
-#include <time.h>
-#include "ftlib.h"
-#include "fmt.h"
 
 #define FT_OFFSET(A,B)\
   o->A = offsetof(struct B, A);
@@ -53,18 +51,26 @@
 void ftrec_xlate_1to5(struct fts3rec_v1 *rec_v1, struct fts3rec_v5 *rec_v5);
 void ftrec_xlate_1to6(struct fts3rec_v1 *rec_v1, struct fts3rec_v6 *rec_v6);
 void ftrec_xlate_1to7(struct fts3rec_v1 *rec_v1, struct fts3rec_v7 *rec_v7);
+void ftrec_xlate_1to1005(struct fts3rec_v1 *rec_v1,
+  struct fts3rec_v1005 *rec_v1005);
 
 void ftrec_xlate_5to1(struct fts3rec_v5 *rec_v5, struct fts3rec_v1 *rec_v1);
 void ftrec_xlate_5to6(struct fts3rec_v5 *rec_v5, struct fts3rec_v6 *rec_v6);
 void ftrec_xlate_5to7(struct fts3rec_v5 *rec_v5, struct fts3rec_v7 *rec_v7);
+void ftrec_xlate_5to1005(struct fts3rec_v5 *rec_v5,
+  struct fts3rec_v1005 *rec_v1005);
 
 void ftrec_xlate_6to1(struct fts3rec_v6 *rec_v6, struct fts3rec_v1 *rec_v1);
 void ftrec_xlate_6to5(struct fts3rec_v6 *rec_v6, struct fts3rec_v5 *rec_v5);
 void ftrec_xlate_6to7(struct fts3rec_v6 *rec_v6, struct fts3rec_v7 *rec_v7);
+void ftrec_xlate_6to1005(struct fts3rec_v6 *rec_v6,
+  struct fts3rec_v1005 *rec_v1005);
 
 void ftrec_xlate_7to1(struct fts3rec_v7 *rec_v7, struct fts3rec_v1 *rec_v1);
 void ftrec_xlate_7to5(struct fts3rec_v7 *rec_v7, struct fts3rec_v5 *rec_v5);
 void ftrec_xlate_7to6(struct fts3rec_v7 *rec_v7, struct fts3rec_v6 *rec_v6);
+void ftrec_xlate_7to1005(struct fts3rec_v7 *rec_v7,
+  struct fts3rec_v1005 *rec_v1005);
 
 /*
  * function ftrec_compute_mask
@@ -127,6 +133,7 @@ int ftrec_mask_ip(void *rec, struct ftver *ftv, struct ftipmask *m)
     case 5:
     case 6:
     case 7:
+    case 1005:
       rec_gen = rec;
       if (!((rec_gen->dstaddr & m->mcast_mask) == m->mcast_val)) {
         rec_gen->srcaddr &= m->src_mask;
@@ -160,6 +167,8 @@ void *ftrec_xlate_func(struct ftver *in_ftv, struct ftver *out_ftv)
       return ftrec_xlate_1to7;
     else if (out_ftv->d_version == 1)
       return (void*)0L;
+    else if (out_ftv->d_version == 1005)
+      return ftrec_xlate_5to1005;
 
   } else if (in_ftv->d_version == 5) {
 
@@ -169,6 +178,8 @@ void *ftrec_xlate_func(struct ftver *in_ftv, struct ftver *out_ftv)
       return ftrec_xlate_5to6;
     else if (out_ftv->d_version == 7)
       return ftrec_xlate_5to7;
+    else if (out_ftv->d_version == 1005)
+      return ftrec_xlate_5to1005;
     else if (out_ftv->d_version == 5)
       return (void*)0L;
 
@@ -182,6 +193,8 @@ void *ftrec_xlate_func(struct ftver *in_ftv, struct ftver *out_ftv)
       return ftrec_xlate_6to7;
     else if (out_ftv->d_version == 6)
       return (void*)0L;
+    else if (out_ftv->d_version == 1005)
+      return ftrec_xlate_6to1005;
 
   } else if (in_ftv->d_version == 7) {
 
@@ -193,6 +206,8 @@ void *ftrec_xlate_func(struct ftver *in_ftv, struct ftver *out_ftv)
       return ftrec_xlate_7to6;
     else if (out_ftv->d_version == 7)
       return (void*)0L;
+    else if (out_ftv->d_version == 1005)
+      return ftrec_xlate_7to1005;
 
   }
 
@@ -275,6 +290,40 @@ void ftrec_xlate_5to6(struct fts3rec_v5 *rec_v5, struct fts3rec_v6 *rec_v6)
 
 } /* ftrec_xlate_5to6 */
 
+void ftrec_xlate_5to1005(struct fts3rec_v5 *rec_v5,
+  struct fts3rec_v1005 *rec_v1005)
+{
+
+  bzero(rec_v1005, sizeof (struct fts3rec_v1005));
+
+  rec_v1005->unix_secs = rec_v5->unix_secs;
+  rec_v1005->unix_nsecs = rec_v5->unix_nsecs;
+  rec_v1005->sysUpTime = rec_v5->sysUpTime;
+  rec_v1005->exaddr = rec_v5->exaddr;
+  rec_v1005->srcaddr = rec_v5->srcaddr;
+  rec_v1005->dstaddr = rec_v5->dstaddr;
+  rec_v1005->nexthop = rec_v5->nexthop;
+  rec_v1005->input = rec_v5->input;
+  rec_v1005->output = rec_v5->output;
+  rec_v1005->dPkts = rec_v5->dPkts;
+  rec_v1005->dOctets = rec_v5->dOctets;
+  rec_v1005->First = rec_v5->First;
+  rec_v1005->Last = rec_v5->Last;
+  rec_v1005->srcport = rec_v5->srcport;
+  rec_v1005->dstport = rec_v5->dstport;
+  rec_v1005->prot = rec_v5->prot;
+  rec_v1005->tos = rec_v5->tos;
+  rec_v1005->tcp_flags = rec_v5->tcp_flags;
+  rec_v1005->engine_type = rec_v5->engine_type;
+  rec_v1005->engine_id = rec_v5->engine_id;
+  rec_v1005->src_mask = rec_v5->src_mask;
+  rec_v1005->dst_mask = rec_v5->dst_mask;
+  rec_v1005->src_as = rec_v5->src_as;
+  rec_v1005->dst_as = rec_v5->dst_as;
+
+} /* ftrec_xlate_5to1005 */
+
+
 void ftrec_xlate_5to7(struct fts3rec_v5 *rec_v5, struct fts3rec_v7 *rec_v7)
 {
 
@@ -387,6 +436,33 @@ void ftrec_xlate_1to7(struct fts3rec_v1 *rec_v1, struct fts3rec_v7 *rec_v7)
 
 } /* ftrec_xlate_1to7 */
 
+void ftrec_xlate_1to1005(struct fts3rec_v1 *rec_v1,
+  struct fts3rec_v1005 *rec_v1005)
+{
+
+  bzero(rec_v1005, sizeof (struct fts3rec_v1005));
+
+  rec_v1005->unix_secs = rec_v1->unix_secs;
+  rec_v1005->unix_nsecs = rec_v1->unix_nsecs;
+  rec_v1005->sysUpTime = rec_v1->sysUpTime;
+  rec_v1005->exaddr = rec_v1->exaddr;
+  rec_v1005->srcaddr = rec_v1->srcaddr;
+  rec_v1005->dstaddr = rec_v1->dstaddr;
+  rec_v1005->nexthop = rec_v1->nexthop;
+  rec_v1005->input = rec_v1->input;
+  rec_v1005->output = rec_v1->output;
+  rec_v1005->dPkts = rec_v1->dPkts;
+  rec_v1005->dOctets = rec_v1->dOctets;
+  rec_v1005->First = rec_v1->First;
+  rec_v1005->Last = rec_v1->Last;
+  rec_v1005->srcport = rec_v1->srcport;
+  rec_v1005->dstport = rec_v1->dstport;
+  rec_v1005->prot = rec_v1->prot;
+  rec_v1005->tos = rec_v1->tos;
+  rec_v1005->tcp_flags = rec_v1->tcp_flags;
+
+} /* ftrec_xlate_1to1005 */
+
 void ftrec_xlate_6to1(struct fts3rec_v6 *rec_v6, struct fts3rec_v1 *rec_v1)
 {
 
@@ -477,6 +553,39 @@ void ftrec_xlate_6to7(struct fts3rec_v6 *rec_v6, struct fts3rec_v7 *rec_v7)
 
 } /* ftrec_xlate_6to7 */
 
+void ftrec_xlate_6to1005(struct fts3rec_v6 *rec_v6,
+  struct fts3rec_v1005 *rec_v1005)
+{
+
+  bzero(rec_v1005, sizeof (struct fts3rec_v1005));
+
+  rec_v1005->unix_secs = rec_v6->unix_secs;
+  rec_v1005->unix_nsecs = rec_v6->unix_nsecs;
+  rec_v1005->sysUpTime = rec_v6->sysUpTime;
+  rec_v1005->exaddr = rec_v6->exaddr;
+  rec_v1005->srcaddr = rec_v6->srcaddr;
+  rec_v1005->dstaddr = rec_v6->dstaddr;
+  rec_v1005->nexthop = rec_v6->nexthop;
+  rec_v1005->input = rec_v6->input;
+  rec_v1005->output = rec_v6->output;
+  rec_v1005->dPkts = rec_v6->dPkts;
+  rec_v1005->dOctets = rec_v6->dOctets;
+  rec_v1005->First = rec_v6->First;
+  rec_v1005->Last = rec_v6->Last;
+  rec_v1005->srcport = rec_v6->srcport;
+  rec_v1005->dstport = rec_v6->dstport;
+  rec_v1005->prot = rec_v6->prot;
+  rec_v1005->tos = rec_v6->tos;
+  rec_v1005->tcp_flags = rec_v6->tcp_flags;
+  rec_v1005->engine_type = rec_v6->engine_type;
+  rec_v1005->engine_id = rec_v6->engine_id;
+  rec_v1005->src_mask = rec_v6->src_mask;
+  rec_v1005->dst_mask = rec_v6->dst_mask;
+  rec_v1005->src_as = rec_v6->src_as;
+  rec_v1005->dst_as = rec_v6->dst_as;
+
+} /* ftrec_xlate_6to1005 */
+
 void ftrec_xlate_7to1(struct fts3rec_v7 *rec_v7, struct fts3rec_v1 *rec_v1)
 {
 
@@ -568,6 +677,39 @@ void ftrec_xlate_7to6(struct fts3rec_v7 *rec_v7, struct fts3rec_v6 *rec_v6)
 
 } /* ftrec_xlate_7to6 */
 
+void ftrec_xlate_7to1005(struct fts3rec_v7 *rec_v7,
+  struct fts3rec_v1005 *rec_v1005)
+{
+
+  bzero(rec_v1005, sizeof (struct fts3rec_v1005));
+
+  rec_v1005->unix_secs = rec_v7->unix_secs;
+  rec_v1005->unix_nsecs = rec_v7->unix_nsecs;
+  rec_v1005->sysUpTime = rec_v7->sysUpTime;
+  rec_v1005->exaddr = rec_v7->exaddr;
+  rec_v1005->srcaddr = rec_v7->srcaddr;
+  rec_v1005->dstaddr = rec_v7->dstaddr;
+  rec_v1005->nexthop = rec_v7->nexthop;
+  rec_v1005->input = rec_v7->input;
+  rec_v1005->output = rec_v7->output;
+  rec_v1005->dPkts = rec_v7->dPkts;
+  rec_v1005->dOctets = rec_v7->dOctets;
+  rec_v1005->First = rec_v7->First;
+  rec_v1005->Last = rec_v7->Last;
+  rec_v1005->srcport = rec_v7->srcport;
+  rec_v1005->dstport = rec_v7->dstport;
+  rec_v1005->prot = rec_v7->prot;
+  rec_v1005->tos = rec_v7->tos;
+  rec_v1005->tcp_flags = rec_v7->tcp_flags;
+  rec_v1005->engine_type = rec_v7->engine_type;
+  rec_v1005->engine_id = rec_v7->engine_id;
+  rec_v1005->src_mask = rec_v7->src_mask;
+  rec_v1005->dst_mask = rec_v7->dst_mask;
+  rec_v1005->src_as = rec_v7->src_as;
+  rec_v1005->dst_as = rec_v7->dst_as;
+
+} /* ftrec_xlate_7to1005 */
+
 /*
  * function fts3rec_compute_offsets
  *
@@ -745,13 +887,12 @@ int fts3rec_compute_offsets(struct fts3rec_offsets *o, struct ftver *ftv)
           FT_OFFSET(dPkts, fts3rec_v8_3)
           FT_OFFSET(First, fts3rec_v8_3)
           FT_OFFSET(Last, fts3rec_v8_3)
-          FT_OFFSET(src_prefix, fts3rec_v8_3)
+          FT_OFFSET(srcaddr, fts3rec_v8_3)
           FT_OFFSET(src_mask, fts3rec_v8_3)
           FT_OFFSET(src_as, fts3rec_v8_3)
           FT_OFFSET(input, fts3rec_v8_3)
           FT_OFFSET(engine_id, fts3rec_v8_3)
           FT_OFFSET(engine_type, fts3rec_v8_3)
-          o->srcaddr = o->src_prefix;
           ret = 0;
           break;
 
@@ -765,13 +906,12 @@ int fts3rec_compute_offsets(struct fts3rec_offsets *o, struct ftver *ftv)
           FT_OFFSET(dPkts, fts3rec_v8_4)
           FT_OFFSET(First, fts3rec_v8_4)
           FT_OFFSET(Last, fts3rec_v8_4)
-          FT_OFFSET(dst_prefix, fts3rec_v8_4)
+          FT_OFFSET(dstaddr, fts3rec_v8_4)
           FT_OFFSET(dst_mask, fts3rec_v8_4)
           FT_OFFSET(dst_as, fts3rec_v8_4)
           FT_OFFSET(output, fts3rec_v8_4)
           FT_OFFSET(engine_id, fts3rec_v8_4)
           FT_OFFSET(engine_type, fts3rec_v8_4)
-          o->dstaddr = o->dst_prefix;
           ret = 0;
           break;
 
@@ -785,26 +925,256 @@ int fts3rec_compute_offsets(struct fts3rec_offsets *o, struct ftver *ftv)
           FT_OFFSET(dPkts, fts3rec_v8_5)
           FT_OFFSET(First, fts3rec_v8_5)
           FT_OFFSET(Last, fts3rec_v8_5)
-          FT_OFFSET(src_prefix, fts3rec_v8_5)
+          FT_OFFSET(srcaddr, fts3rec_v8_5)
           FT_OFFSET(src_mask, fts3rec_v8_5)
           FT_OFFSET(src_as, fts3rec_v8_5)
-          FT_OFFSET(dst_prefix, fts3rec_v8_5)
+          FT_OFFSET(dstaddr, fts3rec_v8_5)
           FT_OFFSET(dst_mask, fts3rec_v8_5)
           FT_OFFSET(dst_as, fts3rec_v8_5)
           FT_OFFSET(input, fts3rec_v8_5)
           FT_OFFSET(output, fts3rec_v8_5)
           FT_OFFSET(engine_id, fts3rec_v8_5)
           FT_OFFSET(engine_type, fts3rec_v8_5)
-          o->dstaddr = o->dst_prefix;
-          o->srcaddr = o->src_prefix;
+          ret = 0;
+          break;
+
+        case 6:
+          FT_OFFSET(unix_secs, fts3rec_v8_6)
+          FT_OFFSET(unix_nsecs, fts3rec_v8_6)
+          FT_OFFSET(sysUpTime, fts3rec_v8_6)
+          FT_OFFSET(exaddr, fts3rec_v8_6)
+          FT_OFFSET(dPkts, fts3rec_v8_6)
+          FT_OFFSET(dOctets, fts3rec_v8_6)
+          FT_OFFSET(First, fts3rec_v8_6)
+          FT_OFFSET(Last, fts3rec_v8_6)
+          FT_OFFSET(dstaddr, fts3rec_v8_6)
+          FT_OFFSET(extra_pkts, fts3rec_v8_6)
+          FT_OFFSET(router_sc, fts3rec_v8_6)
+          FT_OFFSET(output, fts3rec_v8_6)
+          FT_OFFSET(tos, fts3rec_v8_6)
+          FT_OFFSET(marked_tos, fts3rec_v8_6)
+          FT_OFFSET(engine_type, fts3rec_v8_6)
+          FT_OFFSET(engine_id, fts3rec_v8_6)
+          ret = 0;
+          break;
+
+        case 7:
+          FT_OFFSET(unix_secs, fts3rec_v8_7)
+          FT_OFFSET(unix_nsecs, fts3rec_v8_7)
+          FT_OFFSET(sysUpTime, fts3rec_v8_7)
+          FT_OFFSET(exaddr, fts3rec_v8_7)
+          FT_OFFSET(dPkts, fts3rec_v8_7)
+          FT_OFFSET(dOctets, fts3rec_v8_7)
+          FT_OFFSET(First, fts3rec_v8_7)
+          FT_OFFSET(Last, fts3rec_v8_7)
+          FT_OFFSET(dstaddr, fts3rec_v8_7)
+          FT_OFFSET(srcaddr, fts3rec_v8_7)
+          FT_OFFSET(extra_pkts, fts3rec_v8_7)
+          FT_OFFSET(router_sc, fts3rec_v8_7)
+          FT_OFFSET(output, fts3rec_v8_7)
+          FT_OFFSET(input, fts3rec_v8_7)
+          FT_OFFSET(tos, fts3rec_v8_7)
+          FT_OFFSET(marked_tos, fts3rec_v8_7)
+          FT_OFFSET(engine_type, fts3rec_v8_7)
+          FT_OFFSET(engine_id, fts3rec_v8_7)
+          ret = 0;
+          break;
+
+        case 8:
+          FT_OFFSET(unix_secs, fts3rec_v8_8)
+          FT_OFFSET(unix_nsecs, fts3rec_v8_8)
+          FT_OFFSET(sysUpTime, fts3rec_v8_8)
+          FT_OFFSET(exaddr, fts3rec_v8_8)
+          FT_OFFSET(dPkts, fts3rec_v8_8)
+          FT_OFFSET(dOctets, fts3rec_v8_8)
+          FT_OFFSET(First, fts3rec_v8_8)
+          FT_OFFSET(Last, fts3rec_v8_8)
+          FT_OFFSET(dstaddr, fts3rec_v8_8)
+          FT_OFFSET(srcaddr, fts3rec_v8_8)
+          FT_OFFSET(extra_pkts, fts3rec_v8_8)
+          FT_OFFSET(router_sc, fts3rec_v8_8)
+          FT_OFFSET(srcport, fts3rec_v8_8)
+          FT_OFFSET(dstport, fts3rec_v8_8)
+          FT_OFFSET(output, fts3rec_v8_8)
+          FT_OFFSET(input, fts3rec_v8_8)
+          FT_OFFSET(tos, fts3rec_v8_8)
+          FT_OFFSET(marked_tos, fts3rec_v8_8)
+          FT_OFFSET(engine_type, fts3rec_v8_8)
+          FT_OFFSET(engine_id, fts3rec_v8_8)
+          FT_OFFSET(prot, fts3rec_v8_8)
+          ret = 0;
+          break;
+
+        case 9:
+          FT_OFFSET(unix_secs, fts3rec_v8_9)
+          FT_OFFSET(unix_nsecs, fts3rec_v8_9)
+          FT_OFFSET(sysUpTime, fts3rec_v8_9)
+          FT_OFFSET(exaddr, fts3rec_v8_9)
+          FT_OFFSET(dFlows, fts3rec_v8_9)
+          FT_OFFSET(dPkts, fts3rec_v8_9)
+          FT_OFFSET(dOctets, fts3rec_v8_9)
+          FT_OFFSET(First, fts3rec_v8_9)
+          FT_OFFSET(Last, fts3rec_v8_9)
+          FT_OFFSET(src_as, fts3rec_v8_9)
+          FT_OFFSET(dst_as, fts3rec_v8_9)
+          FT_OFFSET(input, fts3rec_v8_9)
+          FT_OFFSET(output, fts3rec_v8_9)
+          FT_OFFSET(engine_type, fts3rec_v8_9)
+          FT_OFFSET(engine_id, fts3rec_v8_9)
+          FT_OFFSET(tos, fts3rec_v8_9)
+          ret = 0;
+          break;
+
+        case 10:
+          FT_OFFSET(unix_secs, fts3rec_v8_10)
+          FT_OFFSET(unix_nsecs, fts3rec_v8_10)
+          FT_OFFSET(sysUpTime, fts3rec_v8_10)
+          FT_OFFSET(exaddr, fts3rec_v8_10)
+          FT_OFFSET(dFlows, fts3rec_v8_10)
+          FT_OFFSET(dPkts, fts3rec_v8_10)
+          FT_OFFSET(dOctets, fts3rec_v8_10)
+          FT_OFFSET(First, fts3rec_v8_10)
+          FT_OFFSET(Last, fts3rec_v8_10)
+          FT_OFFSET(srcport, fts3rec_v8_10)
+          FT_OFFSET(dstport, fts3rec_v8_10)
+          FT_OFFSET(input, fts3rec_v8_10)
+          FT_OFFSET(output, fts3rec_v8_10)
+          FT_OFFSET(engine_type, fts3rec_v8_10)
+          FT_OFFSET(engine_id, fts3rec_v8_10)
+          FT_OFFSET(prot, fts3rec_v8_10)
+          FT_OFFSET(tos, fts3rec_v8_10)
+          ret = 0;
+          break;
+
+        case 11:
+          FT_OFFSET(unix_secs, fts3rec_v8_11)
+          FT_OFFSET(unix_nsecs, fts3rec_v8_11)
+          FT_OFFSET(sysUpTime, fts3rec_v8_11)
+          FT_OFFSET(exaddr, fts3rec_v8_11)
+          FT_OFFSET(dFlows, fts3rec_v8_11)
+          FT_OFFSET(dPkts, fts3rec_v8_11)
+          FT_OFFSET(dOctets, fts3rec_v8_11)
+          FT_OFFSET(First, fts3rec_v8_11)
+          FT_OFFSET(Last, fts3rec_v8_11)
+          FT_OFFSET(srcaddr, fts3rec_v8_11)
+          FT_OFFSET(src_mask, fts3rec_v8_11)
+          FT_OFFSET(tos, fts3rec_v8_11)
+          FT_OFFSET(src_as, fts3rec_v8_11)
+          FT_OFFSET(input, fts3rec_v8_11)
+          FT_OFFSET(engine_type, fts3rec_v8_11)
+          FT_OFFSET(engine_id, fts3rec_v8_11)
+          ret = 0;
+          break;
+
+        case 12:
+          FT_OFFSET(unix_secs, fts3rec_v8_12)
+          FT_OFFSET(unix_nsecs, fts3rec_v8_12)
+          FT_OFFSET(sysUpTime, fts3rec_v8_12)
+          FT_OFFSET(exaddr, fts3rec_v8_12)
+          FT_OFFSET(dFlows, fts3rec_v8_12)
+          FT_OFFSET(dPkts, fts3rec_v8_12)
+          FT_OFFSET(dOctets, fts3rec_v8_12)
+          FT_OFFSET(First, fts3rec_v8_12)
+          FT_OFFSET(Last, fts3rec_v8_12)
+          FT_OFFSET(dstaddr, fts3rec_v8_12)
+          FT_OFFSET(output, fts3rec_v8_12)
+          FT_OFFSET(dst_as, fts3rec_v8_12)
+          FT_OFFSET(dst_mask, fts3rec_v8_12)
+          FT_OFFSET(tos, fts3rec_v8_12)
+          FT_OFFSET(engine_type, fts3rec_v8_12)
+          FT_OFFSET(engine_id, fts3rec_v8_12)
+          ret = 0;
+          break;
+
+        case 13:
+          FT_OFFSET(unix_secs, fts3rec_v8_13)
+          FT_OFFSET(unix_nsecs, fts3rec_v8_13)
+          FT_OFFSET(sysUpTime, fts3rec_v8_13)
+          FT_OFFSET(exaddr, fts3rec_v8_13)
+          FT_OFFSET(dFlows, fts3rec_v8_13)
+          FT_OFFSET(dPkts, fts3rec_v8_13)
+          FT_OFFSET(dOctets, fts3rec_v8_13)
+          FT_OFFSET(First, fts3rec_v8_13)
+          FT_OFFSET(Last, fts3rec_v8_13)
+          FT_OFFSET(srcaddr, fts3rec_v8_13)
+          FT_OFFSET(dstaddr, fts3rec_v8_13)
+          FT_OFFSET(src_as, fts3rec_v8_13)
+          FT_OFFSET(dst_as, fts3rec_v8_13)
+          FT_OFFSET(input, fts3rec_v8_13)
+          FT_OFFSET(output, fts3rec_v8_13)
+          FT_OFFSET(dst_mask, fts3rec_v8_13)
+          FT_OFFSET(src_mask, fts3rec_v8_13)
+          FT_OFFSET(engine_type, fts3rec_v8_13)
+          FT_OFFSET(engine_id, fts3rec_v8_13)
+          FT_OFFSET(tos, fts3rec_v8_13)
+          ret = 0;
+          break;
+
+        case 14:
+          FT_OFFSET(unix_secs, fts3rec_v8_14)
+          FT_OFFSET(unix_nsecs, fts3rec_v8_14)
+          FT_OFFSET(sysUpTime, fts3rec_v8_14)
+          FT_OFFSET(exaddr, fts3rec_v8_14)
+          FT_OFFSET(dFlows, fts3rec_v8_14)
+          FT_OFFSET(dPkts, fts3rec_v8_14)
+          FT_OFFSET(dOctets, fts3rec_v8_14)
+          FT_OFFSET(First, fts3rec_v8_14)
+          FT_OFFSET(Last, fts3rec_v8_14)
+          FT_OFFSET(srcaddr, fts3rec_v8_14)
+          FT_OFFSET(dstaddr, fts3rec_v8_14)
+          FT_OFFSET(srcport, fts3rec_v8_14)
+          FT_OFFSET(dstport, fts3rec_v8_14)
+          FT_OFFSET(input, fts3rec_v8_14)
+          FT_OFFSET(output, fts3rec_v8_14)
+          FT_OFFSET(dst_mask, fts3rec_v8_14)
+          FT_OFFSET(src_mask, fts3rec_v8_14)
+          FT_OFFSET(engine_type, fts3rec_v8_14)
+          FT_OFFSET(engine_id, fts3rec_v8_14)
+          FT_OFFSET(tos, fts3rec_v8_14)
+          FT_OFFSET(prot, fts3rec_v8_14)
           ret = 0;
           break;
 
       }
       break;
 
+    case 1005:
+      FT_OFFSET(unix_secs, fts3rec_v1005)
+      FT_OFFSET(unix_nsecs, fts3rec_v1005)
+      FT_OFFSET(sysUpTime, fts3rec_v1005)
+      FT_OFFSET(exaddr, fts3rec_v1005)
+      FT_OFFSET(srcaddr, fts3rec_v1005)
+      FT_OFFSET(dstaddr, fts3rec_v1005)
+      FT_OFFSET(nexthop, fts3rec_v1005)
+      FT_OFFSET(input, fts3rec_v1005)
+      FT_OFFSET(output, fts3rec_v1005)
+      FT_OFFSET(dPkts, fts3rec_v1005)
+      FT_OFFSET(dOctets, fts3rec_v1005)
+      FT_OFFSET(First, fts3rec_v1005)
+      FT_OFFSET(Last, fts3rec_v1005)
+      FT_OFFSET(srcport, fts3rec_v1005)
+      FT_OFFSET(dstport, fts3rec_v1005)
+      FT_OFFSET(prot, fts3rec_v1005)
+      FT_OFFSET(tos, fts3rec_v1005)
+      FT_OFFSET(tcp_flags, fts3rec_v1005)
+      FT_OFFSET(engine_type, fts3rec_v1005)
+      FT_OFFSET(engine_id, fts3rec_v1005)
+      FT_OFFSET(src_mask, fts3rec_v1005)
+      FT_OFFSET(dst_mask, fts3rec_v1005)
+      FT_OFFSET(src_as, fts3rec_v1005)
+      FT_OFFSET(dst_as, fts3rec_v1005)
+      FT_OFFSET(src_tag, fts3rec_v1005)
+      FT_OFFSET(dst_tag, fts3rec_v1005)
+      ret = 0;
+      break;
+
   } /* switch */
 
+  o->xfields = ftrec_xfield(ftv);
+
+  bcopy(ftv, &o->ftv, sizeof *ftv);
+
   return ret;
 
 } /* fts3rec_compute_offsets */
+
