@@ -1,6 +1,6 @@
 #ifndef lint
 static char const 
-yyrcsid[] = "$FreeBSD: src/usr.bin/yacc/skeleton.c,v 1.28 2000/01/17 02:04:06 bde Exp $";
+yyrcsid[] = "$FreeBSD: src/usr.bin/yacc/skeleton.c,v 1.28.2.1 2001/07/19 05:46:39 peter Exp $";
 #endif
 #include <stdlib.h>
 #define YYBYACC 1
@@ -11,11 +11,17 @@ yyrcsid[] = "$FreeBSD: src/usr.bin/yacc/skeleton.c,v 1.28 2000/01/17 02:04:06 bd
 #define yyclearin (yychar=(YYEMPTY))
 #define yyerrok (yyerrflag=0)
 #define YYRECOVERING() (yyerrflag!=0)
+#if defined(__cplusplus) || __STDC__
+static int yygrowstack(void);
+#else
 static int yygrowstack();
+#endif
 #define YYPREFIX "yy"
 #line 2 "aclyacc.y"
-#include "bytes.h"
-#include "fmt.h"
+
+#include "ftconfig.h"
+#include "ftlib.h"
+
 #include "acl2.h"
 #include <stdio.h> /* XXX REMOVE */
 
@@ -27,7 +33,7 @@ extern struct acl_list acl_list;
 
 int x;
 
-#line 17 "aclyacc.y"
+#line 16 "aclyacc.y"
 typedef union {
 	char *c;
 	u_long long ip;
@@ -36,7 +42,7 @@ typedef union {
 	struct acl_ip_std_entry std_entry;
 	struct acl_ip_ext_entry ext_entry;
 } YYSTYPE;
-#line 40 "y.tab.c"
+#line 43 "y.tab.c"
 #define YYERRCODE 256
 #define HOST 257
 #define ANY 258
@@ -520,7 +526,7 @@ yyreduce:
     switch (yyn)
     {
 case 10:
-#line 55 "aclyacc.y"
+#line 54 "aclyacc.y"
 {
 		/* XXX add code to check NUM is in range 0..99 */
 		yyvsp[0].std_entry.flag |= yyvsp[-1].i;
@@ -529,7 +535,7 @@ case 10:
 	}
 break;
 case 11:
-#line 61 "aclyacc.y"
+#line 60 "aclyacc.y"
 {
 		/* XXX add code to check NUM is in range 100-199 */
 		yyvsp[0].ext_entry.flag |= yyvsp[-1].i;
@@ -538,7 +544,7 @@ case 11:
 	}
 break;
 case 12:
-#line 67 "aclyacc.y"
+#line 66 "aclyacc.y"
 {
 		yyvsp[0].std_entry.flag |= yyvsp[-1].i;
 		x = acl_create(&acl_list, yyvsp[-2].c, ACL_TYPE_STD);
@@ -546,7 +552,7 @@ case 12:
 	}
 break;
 case 13:
-#line 72 "aclyacc.y"
+#line 71 "aclyacc.y"
 {
 		yyvsp[0].ext_entry.flag |= yyvsp[-1].i;
 		x = acl_create(&acl_list, yyvsp[-2].c, ACL_TYPE_EXT);
@@ -554,7 +560,7 @@ case 13:
 	}
 break;
 case 14:
-#line 80 "aclyacc.y"
+#line 79 "aclyacc.y"
 {
 		yyval.std_entry.src_addr = 0x0;
 		yyval.std_entry.src_mask = 0xFFFFFFFF;
@@ -563,7 +569,7 @@ case 14:
 	}
 break;
 case 15:
-#line 86 "aclyacc.y"
+#line 85 "aclyacc.y"
 { 
 		yyval.std_entry.src_addr = yyvsp[0].ip;
 		yyval.std_entry.src_mask = 0x0;
@@ -572,7 +578,7 @@ case 15:
 	}
 break;
 case 16:
-#line 92 "aclyacc.y"
+#line 91 "aclyacc.y"
 { 
 		yyval.std_entry.src_addr = yyvsp[-1].ip;
 		yyval.std_entry.src_mask = yyvsp[0].ip;
@@ -581,7 +587,7 @@ case 16:
 	}
 break;
 case 17:
-#line 98 "aclyacc.y"
+#line 97 "aclyacc.y"
 {
 		yyval.std_entry.src_addr = yyvsp[0].ip;
 		yyval.std_entry.src_mask = 0x0;
@@ -590,19 +596,19 @@ case 17:
 	}
 break;
 case 18:
-#line 106 "aclyacc.y"
+#line 105 "aclyacc.y"
 {
 		yyval.i = ACL_FLAG_PERMIT;
 	}
 break;
 case 19:
-#line 109 "aclyacc.y"
+#line 108 "aclyacc.y"
 {
 		yyval.i = 0;
 	}
 break;
 case 25:
-#line 122 "aclyacc.y"
+#line 121 "aclyacc.y"
 {
 		yyval.ext_entry.protocol = 0;
 		yyval.ext_entry.precedence = 0;
@@ -625,7 +631,7 @@ case 25:
 	}
 break;
 case 26:
-#line 142 "aclyacc.y"
+#line 141 "aclyacc.y"
 {
 		yyval.ext_entry.protocol = 0;
 		yyval.ext_entry.precedence = yyvsp[0].ext_entry.precedence;
@@ -648,7 +654,7 @@ case 26:
 	}
 break;
 case 27:
-#line 164 "aclyacc.y"
+#line 163 "aclyacc.y"
 {
 		yyval.ext_entry.protocol = IPPROTO_ICMP;
 		yyval.ext_entry.precedence = 0;
@@ -671,19 +677,19 @@ case 27:
 	}
 break;
 case 28:
-#line 184 "aclyacc.y"
+#line 183 "aclyacc.y"
 {}
 break;
 case 29:
-#line 185 "aclyacc.y"
+#line 184 "aclyacc.y"
 {}
 break;
 case 30:
-#line 186 "aclyacc.y"
+#line 185 "aclyacc.y"
 {}
 break;
 case 31:
-#line 189 "aclyacc.y"
+#line 188 "aclyacc.y"
 {
 		yyval.ext_entry.protocol = IPPROTO_IGMP;
 		yyval.ext_entry.precedence = 0;
@@ -706,19 +712,19 @@ case 31:
 	}
 break;
 case 32:
-#line 209 "aclyacc.y"
+#line 208 "aclyacc.y"
 {}
 break;
 case 33:
-#line 210 "aclyacc.y"
+#line 209 "aclyacc.y"
 {}
 break;
 case 34:
-#line 211 "aclyacc.y"
+#line 210 "aclyacc.y"
 {}
 break;
 case 35:
-#line 214 "aclyacc.y"
+#line 213 "aclyacc.y"
 {
 		yyval.ext_entry.protocol = IPPROTO_TCP;
 		yyval.ext_entry.precedence = 0;
@@ -741,19 +747,19 @@ case 35:
 	}
 break;
 case 36:
-#line 234 "aclyacc.y"
+#line 233 "aclyacc.y"
 {}
 break;
 case 37:
-#line 235 "aclyacc.y"
+#line 234 "aclyacc.y"
 {}
 break;
 case 38:
-#line 236 "aclyacc.y"
+#line 235 "aclyacc.y"
 {}
 break;
 case 39:
-#line 239 "aclyacc.y"
+#line 238 "aclyacc.y"
 {
 		yyval.ext_entry.protocol = IPPROTO_UDP;
 		yyval.ext_entry.precedence = 0;
@@ -776,51 +782,51 @@ case 39:
 	}
 break;
 case 40:
-#line 259 "aclyacc.y"
+#line 258 "aclyacc.y"
 {}
 break;
 case 41:
-#line 260 "aclyacc.y"
+#line 259 "aclyacc.y"
 {}
 break;
 case 42:
-#line 261 "aclyacc.y"
+#line 260 "aclyacc.y"
 {}
 break;
 case 61:
-#line 293 "aclyacc.y"
+#line 292 "aclyacc.y"
 {}
 break;
 case 62:
-#line 294 "aclyacc.y"
+#line 293 "aclyacc.y"
 {}
 break;
 case 63:
-#line 295 "aclyacc.y"
+#line 294 "aclyacc.y"
 {}
 break;
 case 64:
-#line 296 "aclyacc.y"
+#line 295 "aclyacc.y"
 {}
 break;
 case 65:
-#line 297 "aclyacc.y"
+#line 296 "aclyacc.y"
 {}
 break;
 case 66:
-#line 298 "aclyacc.y"
+#line 297 "aclyacc.y"
 {}
 break;
 case 67:
-#line 299 "aclyacc.y"
+#line 298 "aclyacc.y"
 {}
 break;
 case 68:
-#line 302 "aclyacc.y"
+#line 301 "aclyacc.y"
 {}
 break;
 case 74:
-#line 316 "aclyacc.y"
+#line 315 "aclyacc.y"
 {
 		yyval.std_entry.src_addr = yyvsp[-1].ip;
 		yyval.std_entry.src_mask = yyvsp[0].ip;
@@ -829,7 +835,7 @@ case 74:
 	}
 break;
 case 75:
-#line 322 "aclyacc.y"
+#line 321 "aclyacc.y"
 {
 		yyval.std_entry.src_addr = 0x0;
 		yyval.std_entry.src_mask = 0xFFFFFFFF;
@@ -838,7 +844,7 @@ case 75:
 	}
 break;
 case 76:
-#line 328 "aclyacc.y"
+#line 327 "aclyacc.y"
 {
 		yyval.std_entry.src_addr = yyvsp[0].ip;
 		yyval.std_entry.src_mask = 0x0;
@@ -846,7 +852,7 @@ case 76:
 		yyval.std_entry.flag = 0;
 	}
 break;
-#line 850 "y.tab.c"
+#line 853 "y.tab.c"
     }
     yyssp -= yym;
     yystate = *yyssp;
