@@ -23,86 +23,73 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- *      $Id: flow-capture.c,v 1.47 2001/07/17 04:13:37 maf Exp $
+ *      $Id: flow-capture.c,v 1.72 2003/02/24 02:55:15 maf Exp $
  */
 
-#if HAVE_CONFIG_H
- #include <config.h>
-#endif
+#include "ftconfig.h"
+#include "ftlib.h"
 
 #include <sys/time.h>
 #include <sys/types.h>
 #include <sys/uio.h>
-#include <sys/errno.h>
 #include <sys/socket.h>
 #include <sys/param.h>
 #include <sys/stat.h>
+#include <sys/wait.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
+#include <errno.h>
+#include <fcntl.h>
 #include <unistd.h>
 #include <signal.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <time.h>
+#include <syslog.h>
+
 #if HAVE_STRINGS_H
  #include <strings.h>
 #endif
+
 #if HAVE_STRING_H
   #include <string.h>
 #endif
-#include <time.h>
-#include <fcntl.h>
-#include <syslog.h>
-#include "ftlib.h"
-#include "fmt.h"
-#include "support.h"
-#ifdef LIBWRAP
+
+#include "ftbuild.h"
+
+#ifdef HAVE_LIBWRAP
 #include <tcpd.h>
-#endif /* LIBWRAP */
-
-
-
-/* TODO
- *
- *  call external program on file rotation
- *
- *  The TCP client write() needs to have userland buffering
- *
- *  fix the client
- *
-*/
+#endif /* HAVE_LIBWRAP */
 
 void fterr_exit_handler(int code);
 
-
 #define CAPTURE_PIDFILE    "/var/run/flow-capture.pid"
 
-#define CLIENT_VALID   1   /* entry in use? */
-#define CLIENT_HEADER  2   /* sent stream header yet? */
-
-#define CLIENT_MAXCON  32  /* max # of TCP connections to server */
-
 #define SELECT_TIMEOUT 1   /* 1 second */
 
 struct client_rec {
   int fd;
   struct sockaddr_in addr;
   int flags;
-  u_int32 con_time;
-#ifdef LIBWRAP
-  struct request_info tcpd;
-#endif /* LIBWRAP */
+  time_t conn_time;
+  struct ftio ftio;
+  FT_LIST_ENTRY (client_rec) chain;
 };
 
 struct client {
-  struct client_rec recs[CLIENT_MAXCON+1]; /* +1 is used for error recovery */
+  FT_LIST_HEAD(clienth, client_rec) list;
   struct sockaddr_in addr;
   int fd;
-  int len;
-  int num;
+  int enabled; /* listen socket enabled? */
+  int max; /* max connections */
+  int active; /* active connections */
+#ifdef HAVE_LIBWRAP
+  struct request_info tcpd;
+#endif /* HAVE_LIBWRAP */
 };
 
 struct rotate {
-  u_int32 next;    /* time of next rotation */
+  double next;    /* time of next rotation */
   int    cur;     /* current rotation # */
   int    n;       /* number per day */
 };
@@ -121,31 +108,32 @@ struct file {
 };
 
 int debug;
-int sig_pipe_flag, sig_quit_flag, sig_hup_flag;
-void sig_pipe();
-void sig_quit();
-void sig_hup();
-char *progname = "flow-capture\0";
+int sig_pipe_flag, sig_quit_flag, sig_hup_flag, sig_chld_flag;
+int reload_flag;
+void sig_pipe(int);
+void sig_quit(int);
+void sig_hup(int);
+void sig_chld(int);
 pid_t pid;
+char *pidfile;
 struct ftnet ftnet;
 
-void usage();
-int calc_rotate (int next, u_int32 *trotate, int *cur);
+void usage(void);
+int calc_rotate (int next, double *trotate, int *cur);
+double doubletime(void);
 
 int main(argc, argv)   
 int argc;
 char **argv;
 {
-  extern int errno;
-  extern char *optarg;
 #ifdef IP_ADD_MEMBERSHIP
-  struct sockaddr_in tmp_addr;
   struct ip_mreq mr;
 #ifdef IP_ADD_SOURCE_MEMBERSHIP
   struct ip_mreq_source mrs;
 #endif
 #endif   
   fd_set rfd;
+  struct sockaddr_in tmp_addr;
   struct timeval tv;
   struct tm *tm;
   struct ftset ftset;
@@ -160,26 +148,54 @@ char **argv;
   struct rotate rot;
   struct file cap_file;
   struct ftipmask ftipmask;
-  time_t now;
-  char work_dir[MAXPATHLEN+1];
-  int i, n, len, invalid, enable_unlink, offset, detach, nest, one;
+  struct fttag fttag;
+  struct ftfil ftfil;
+  struct fttag_def *ftd;
+  struct ftfil_def *ftfd;
+  struct fts3rec_offsets fo;
+  struct client_rec *client_rec, *client_rec2;
+  pid_t child_pid;
+  time_t tt_now, time_startup;
+  double now;
+  char work_dir[MAXPATHLEN+1], post_rotate_exec[MAXPATHLEN+1];
+  int i, n, tmp_len, enable_unlink, offset, detach, nest, one, max_fd;
   unsigned int v1, v2;
   u_int32 privacy_mask, hash;
   char fmt_src_ip[32], fmt_dst_ip[32], fmt_dst_port[32];
-  char out_rec[FT_IO_MAXREC];
-  int stat_interval, stat_next;
+  char xl_rec[FT_IO_MAXREC], *out_rec;
+  char *tag_fname, *tag_active;
+  char *filter_fname, *filter_active;
+  int stat_interval, stat_next, child_status;
+  int v_flag;
+
+  time_startup = time((time_t)0L);
 
   bzero (&rot, sizeof rot);
   bzero (&cap_file, sizeof cap_file);
   bzero (&ftnet, sizeof ftnet);
   bzero (&tv, sizeof tv);
   bzero (&work_dir, sizeof work_dir);
+  bzero (&post_rotate_exec, sizeof post_rotate_exec);
   bzero (&fte, sizeof fte);
   bzero (&client, sizeof client);
   bzero (&ftpdu, sizeof ftpdu);
   bzero (&ftv, sizeof ftv);
+  bzero (&fttag, sizeof fttag);
+  bzero (&ftfil, sizeof ftfil);
+  FT_LIST_INIT(&client.list);
   stat_interval = 0;
   stat_next = -1;
+  v_flag = 0;
+  reload_flag = 1;
+
+  tag_fname = FT_PATH_CFG_TAG;
+  tag_active = (char*)0L;
+
+  filter_fname = FT_PATH_CFG_FILTER;
+  filter_active = (char*)0L;
+
+  ftfd = (struct ftfil_def*)0L;
+  ftd = (struct fttag_def*)0L;
 
   /* init fterr */
   fterr_setid(argv[0]);
@@ -216,7 +232,11 @@ char **argv;
   /* year/month/day nesting */
   nest = 3;
 
-  while ((i = getopt(argc, argv, "A:b:c:C:d:De:E:hm:n:N:S:V:w:z:")) != -1)
+  /* pidfile */
+  pidfile = CAPTURE_PIDFILE;
+
+  while ((i = getopt(argc, argv, "A:b:c:C:d:De:E:f:F:hm:n:N:p:S:t:T:V:w:z:R:"))
+    != -1)
   
     switch (i) {
 
@@ -234,9 +254,7 @@ char **argv;
       break;
 
     case 'c': /* client enable */
-      client.num = atoi(optarg);
-      if (client.num > CLIENT_MAXCON)
-        fterr_errx(1, "CLIENT_MAXCON (%d) exceeded", CLIENT_MAXCON);
+      client.max = atoi(optarg);
       break;
 
     case 'C': /* comment field */
@@ -245,15 +263,28 @@ char **argv;
   
     case 'D': /* daemonize */
       detach = 0;
+      pidfile = (char*)0L;
       break;
 
     case 'e': /* expire */
       fte.max_files = atoi(optarg);
+      if (fte.max_files)
+        fte.expiring = 1;
       break;
   
     case 'E': /* expire bytes */
       if ((fte.max_bytes = scan_size(optarg)) == -1)
         fterr_errx(1, "scan_size(): failed");
+      if (fte.max_bytes)
+        fte.expiring = 1;
+      break;
+
+    case 'f': /* filter fname */
+      filter_fname = optarg;
+      break;
+  
+    case 'F': /* filter active */
+      filter_active = optarg;
       break;
         
     case 'd': /* debug */
@@ -271,15 +302,28 @@ char **argv;
 
     case 'n': /* # rotations / day */
       rot.n = atoi(optarg);
-      /* no more than 1 rotation per 5 minutes */
-      if (rot.n > (288-1))
-        fterr_errx(1, "rotations limited to every 5 minutes");
+      /* no more than 1 rotation per minute */
+      if (rot.n > (1440-1))
+        fterr_errx(1, "rotations limited to every 1 minute");
       break;
 
     case 'N': /* nesting level */
       nest = atoi(optarg);
       if ((nest < -3) || (nest > 3))
-        fterr_errx(1, "-3 <= nesting level <= 3\n");
+        fterr_errx(1, "-3 <= nesting level <= 3");
+      break;
+
+    case 'p': /* pidfile */
+      if ((optarg[0] == 0) || ((optarg[0] == '-') && (optarg[1] == 0)))
+        pidfile = (char*)0L;
+      else
+        pidfile = optarg;
+      break;
+
+    case 'R': /* Post rotate exec */
+      if (strlen(optarg) > MAXPATHLEN)
+        fterr_errx(1, "Post rotate argument too long");
+      strcpy(post_rotate_exec,optarg);
       break;
 
     case 'S': /* stat interval */
@@ -287,8 +331,22 @@ char **argv;
       if ((stat_interval < 0) || (stat_interval > 60))
         fterr_errx(1, "Stat interval must be between 0 and 60.");
       break;
+
+    case 't': /* tag filename */
+      tag_fname = optarg;
+      break;
+
+    case 'T': /* active tags */
+      tag_active = optarg;
+      /* required for fttag_eval() */
+      ftv.s_version = FT_IO_SVERSION;
+      ftv.d_version = 1005;
+      ftv.set = 1;
+    
+      break;
       
     case 'V': /* PDU version */
+      v_flag = 1;
       n = sscanf(optarg, "%u.%u", &v1, &v2);
       if (n == 1) {
         ftv.s_version = FT_IO_SVERSION;
@@ -324,8 +382,19 @@ char **argv;
         
     } /* switch */
 
+  /* disable pidfile if not forking */
+  if (!detach)
+    pidfile = (char*)0L;
+
   if ((argc - optind) != 1)
-    fterr_errx(1, "Specify localip/remoteip/port\n");
+    fterr_errx(1, "Specify localip/remoteip/port.");
+
+  /* tagging forces v1005 */
+  if (v_flag && tag_active && (ftv.d_version != 1005))
+    fterr_errx(1, "Must be v1005 with tagging.");
+
+  if (!work_dir[0])
+    fterr_errx(1, "Specify workdir with -w.");
 
   ftpi = scan_peeri(argv[optind]);
 
@@ -339,6 +408,7 @@ char **argv;
 
   ftnet.loc_addr.sin_addr.s_addr = htonl(ftpi.loc_ip);
   ftnet.loc_addr.sin_port = htons(ftnet.dst_port);
+  client.addr.sin_port = htons(ftnet.dst_port);
 
   /* if debugging is enabled do not unlink any files when aging */
   if (debug) 
@@ -351,18 +421,19 @@ char **argv;
     ftrec_compute_mask(&ftipmask, privacy_mask, privacy_mask,
       ftset.byte_order);
 
- /* daemonize */
+  /* daemonize */
   if (detach) {
     if ((pid = fork()) == -1) {
       fterr_err(1, "fork()");
     } else if (pid) {
-      write_pidfile(pid, CAPTURE_PIDFILE, ftnet.dst_port);
+      if (pidfile)
+        write_pidfile(pid, pidfile, ftnet.dst_port);
       exit (0); /* parent */
     }
 
     umask(0022);
     setsid();
-    for (n = 0; n < 16; ++n) /* XXX dynamically get NOFILE */
+    for (n = 0; n < 16; ++n)
       close (n);
 
     /* enable syslog */
@@ -373,43 +444,21 @@ char **argv;
 
   } 
 
-  /* if client attachments are enabled, setup a listener */
-  if (client.num) {
-    
-    /* socket for listen */
-    if ((client.fd = socket(AF_INET, SOCK_STREAM, 0)) < 0)
-      fterr_err(1, "socket()");
-      
-    if (bind(client.fd, (struct sockaddr*)&client.addr, sizeof (client.addr))
-      < 0)
-      fterr_err(1, "bind()");
-      
-    /* listen for new TCP connections */
-    if (listen(client.fd, 5) < 0)
-      fterr_err(1, "listen()");
-
-    /* non blocking */
-    if (fcntl(client.fd, F_SETFL, O_NONBLOCK) < 0)
-      fterr_err(1, "fcntl()");
-
-  } /* clients enabled */
-
   /*
    * configure signal handlers
    */
 
-  if (signal(SIGPIPE, sig_pipe) < 0)
-    fterr_err(1, "signal()");
+  if (mysignal(SIGPIPE, sig_pipe) == SIG_ERR)
+    fterr_err(1, "signal(SIGPIPE)");
 
-  if (signal(SIGHUP, sig_hup) < 0)
-    fterr_err(1, "signal()");
+  if (mysignal(SIGHUP, sig_hup) == SIG_ERR)
+    fterr_err(1, "signal(SIGHUP)");
 
-  if (signal(SIGQUIT, sig_quit) < 0)
-    fterr_err(1, "signal()");
+  if (mysignal(SIGQUIT, sig_quit) == SIG_ERR)
+    fterr_err(1, "signal(SIGQUIT)");
 
-  /* rotations needs to divide the day evenly */
-  if (rot.n && (86400 % (rot.n+1)))
-    fterr_errx(1, "Rotations must divide the day evenly");
+  if (mysignal(SIGCHLD, sig_chld) == SIG_ERR)
+    fterr_err(1, "signal(SIGCHLD)");
 
   /* sandbox */
   if (chdir(work_dir) == -1)
@@ -418,16 +467,18 @@ char **argv;
   /*
    * load directory entries into the file ager
    */
-  if (ftfile_loaddir(&fte, ".", FT_FILE_SORT|FT_FILE_INIT|FT_FILE_CHECKNAMES))
-    fterr_errx(1, "ftfile_scandir(): failed");
+  if (fte.expiring)
+    if (ftfile_loaddir(&fte, ".", FT_FILE_SORT|FT_FILE_INIT|FT_FILE_CHECKNAMES))
+      fterr_errx(1, "ftfile_scandir(): failed");
 
   /* debugging gets a dump of the ager */
   if (debug)
     ftfile_dump(&fte);
 
   /* run the ager once now */
-  if (ftfile_expire(&fte, enable_unlink, (u_int32)0))
-    fterr_errx(1, "ftfile_export(): failed");
+  if (fte.expiring)
+    if (ftfile_expire(&fte, enable_unlink, (u_int32)0))
+      fterr_errx(1, "ftfile_export(): failed");
 
   /* get hostname */
   if (gethostname((char*)&ftset.hnbuf, FT_HOSTNAME_LEN-1) == -1)
@@ -440,7 +491,7 @@ char **argv;
   if ((ftnet.fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
     fterr_err(1, "socket()");
 
-  if (bigsockbuf(ftnet.fd, SO_RCVBUF, 224*1024) < 0)
+  if (bigsockbuf(ftnet.fd, SO_RCVBUF, FT_SO_RCV_BUFSIZE) < 0)
     fterr_err(1, "bigsockbuf()");
 
 /* multicast capable? */
@@ -469,7 +520,7 @@ char **argv;
 
     if (bind(ftnet.fd, (struct sockaddr*)&tmp_addr,
       sizeof(struct sockaddr)) < 0)
-      fterr_err(1, "bind(mcast-rcv)");
+      fterr_err(1, "bind(%s)", inet_ntoa(tmp_addr.sin_addr));
 
 #ifdef IP_ADD_SOURCE_MEMBERSHIP
 
@@ -502,7 +553,7 @@ char **argv;
     /* unicast bind -- multicast support */
     if (bind(ftnet.fd, (struct sockaddr*)&ftnet.loc_addr,
       sizeof(ftnet.loc_addr)) < 0)
-      fterr_err(1, "bind()");
+      fterr_err(1, "bind(%s)", inet_ntoa(ftnet.loc_addr.sin_addr));
 
   } /* not multicast group */
 
@@ -515,7 +566,7 @@ mcast_done:
   /* unicast bind -- no multicast support */
   if (bind(ftnet.fd, (struct sockaddr*)&ftnet.loc_addr,
     sizeof(ftnet.loc_addr)) < 0)
-    fterr_err(1, "bind()");
+    fterr_err(1, "bind(%s)", inet_ntoa(ftnet.loc_addr.sin_addr)));
 
 #endif /* IP_ADD_MEMBERSHIP */
 
@@ -554,13 +605,14 @@ mcast_done:
     FD_ZERO (&rfd);
     FD_SET (ftnet.fd, &rfd);
 
-    if (client.num)
+    if (client.enabled && client.max) {
       FD_SET (client.fd, &rfd);
+      max_fd = (client.fd > ftnet.fd) ? client.fd : ftnet.fd;
+    } else {
+      max_fd = ftnet.fd;
+    }
 
-/* XXX shouldn't this be the max of the clients and ftnet? */
-/* XXX shouldn't and all the client fd's too? */
-
-    if (select (ftnet.fd+1, &rfd, (fd_set *)0, (fd_set *)0, &tv) < 0)  {
+    if (select (max_fd+1, &rfd, (fd_set *)0, (fd_set *)0, &tv) < 0)  {
       if (errno == EINTR) {
         FD_ZERO (&rfd);
       } else {
@@ -571,11 +623,157 @@ mcast_done:
     bzero (&tv, sizeof tv);
     tv.tv_sec = SELECT_TIMEOUT;
 
-    now = time((time_t)0L); 
+    tt_now = now = doubletime();
+
+    /* new TCP client connection ? */
+    if ((client.max) && (FD_ISSET(client.fd, &rfd))) {
+
+      /* too many clients? */
+      if (client.active >= client.max) {
+        /* bye */
+        i = accept(client.fd, (struct sockaddr*)&tmp_addr, &tmp_len);
+        close(i);
+        fterr_warnx("Maximum clients exceeded, rejecting connection from %s",
+        inet_ntoa(tmp_addr.sin_addr));
+        goto skip_client;
+      }
+
+      /* allocate a new client record, fail gracefully */
+      if (!(client_rec = (struct client_rec*)malloc(sizeof *client_rec))) {
+        fterr_warn("malloc()");
+        /* bye */
+        tmp_len = sizeof (struct sockaddr_in);
+        i = accept(client.fd, (struct sockaddr*)&client_rec->addr, &tmp_len);
+        close(i);
+        goto skip_client;
+      }
+
+      bzero(client_rec, sizeof *client_rec);
+
+      /* link in the new record */
+      FT_LIST_INSERT_HEAD(&client.list, client_rec, chain);
+
+      tmp_len = sizeof (struct sockaddr_in);
+      /* accept() the connection */
+      if ((client_rec->fd = accept(client.fd,
+        (struct sockaddr*)&client_rec->addr, &tmp_len)) < 0) {
+        fterr_warn("accept()");
+        FT_LIST_REMOVE(client_rec, chain);
+        free(client_rec);
+        goto skip_client;
+      }
+
+#ifdef HAVE_LIBWRAP
+
+      request_init(&client.tcpd, RQ_DAEMON, "flow-capture-client", RQ_FILE,
+        client_rec->fd, NULL);
+
+      fromhost(&client.tcpd);
+
+      if (!hosts_access(&client.tcpd)) {
+
+        fterr_warnx("client %s refused by libwrap",
+          inet_ntoa(client_rec->addr.sin_addr));
+
+        close(client_rec->fd);
+
+        FT_LIST_REMOVE(client_rec, chain);
+        free(client_rec);
+
+        goto skip_client;
+
+      }
+
+#endif /* HAVE_LIBWRAP */
+      
+      if (bigsockbuf(client_rec->fd, SO_SNDBUF, FT_SO_SND_BUFSIZE) < 0)
+        fterr_warn("bigsockbuf()");
+
+      /* log it */
+      client_rec->conn_time = tt_now;
+      fterr_info("client connect: ip=%s time=%lu",
+        inet_ntoa(client_rec->addr.sin_addr),
+        (unsigned long)client_rec->conn_time);
+
+      /* ftio_init the stream */
+      if (ftio_init(&client_rec->ftio, client_rec->fd,
+        FT_IO_FLAG_NO_SWAP | FT_IO_FLAG_WRITE) < 0) {
+        fterr_warnx("ftio_init(): failed for client");
+        close(client_rec->fd);
+        FT_LIST_REMOVE(client_rec, chain);
+        free(client_rec);
+        goto skip_client;
+      }
+
+      /* set the version information in the io stream */
+      if (ftio_set_ver(&client_rec->ftio, &ftv) < 0)
+        fterr_errx(1, "ftio_set_ver(): failed");
+
+      ftio_set_comment(&client_rec->ftio, ftset.comments);
+      ftio_set_cap_hostname(&client_rec->ftio, ftset.hnbuf);
+      ftio_set_byte_order(&client_rec->ftio, ftset.byte_order);
+      ftio_set_cap_time(&client_rec->ftio, cap_file.time, 0);
+      ftio_set_debug(&client_rec->ftio, debug);
+
+      /* header first */
+      if ((n = ftio_write_header(&client_rec->ftio)) < 0) {
+        fterr_warnx("ftio_write_header(): failed for client");
+        ftio_close(&client_rec->ftio);
+        FT_LIST_REMOVE(client_rec, chain);
+        free(client_rec);
+        goto skip_client;
+      }
+
+      ++client.active;
+
+    } /* new TCP client */
+
+skip_client:
+
+    /* stake the zombies */
+
+    if (sig_chld_flag) {
+
+      /* not again */
+      sig_chld_flag = 0;
+
+      while (1) {
+
+        child_pid = wait3(&child_status, WNOHANG, 0);
+
+        /* no more dead children? */
+        if (!child_pid)
+          break;
+
+        /* no more dead children? */
+        if ((child_pid == -1) && (errno == ECHILD))
+          break;
+
+        if (WIFEXITED(child_status)) {
+
+          if (WEXITSTATUS(child_status))
+            fterr_warnx("Child %d exit_status=%d", (int)child_pid,
+              WEXITSTATUS(child_status));
+
+        } else if (WIFSIGNALED(child_status)) {
+
+          fterr_warnx("Child %d signal=%d", (int)child_pid,
+            WTERMSIG(child_status));
+
+        } else {
+
+          fterr_warnx("PID %d exited status=%d", (int)child_pid,
+            child_status);
+
+        }
+
+      } /* buffy */
+
+    } /* sig_chld_flag */
 
     if (stat_interval) {
 
-      tm = localtime (&now);
+      tm = localtime (&tt_now);
 
       /*
        * note there is an obscure race condition here if this
@@ -594,11 +792,12 @@ mcast_done:
           fmt_ipv4(fmt_dst_ip, ftch_recexpp->dst_ip, FMT_JUST_LEFT);
 
           fterr_info(
-            "STAT: now=%lu src_ip=%s dst_ip=%s d_ver=%d pkts=%lu flows=%lu lost=%lu reset=%lu",
-            (unsigned long)now, fmt_src_ip, fmt_dst_ip,
+            "STAT: now=%lu startup=%lu src_ip=%s dst_ip=%s d_ver=%d pkts=%lu flows=%lu lost=%lu reset=%lu filter_drops=%lu",
+            (unsigned long)tt_now, (unsigned long)time_startup,
+            fmt_src_ip, fmt_dst_ip,
             ftch_recexpp->d_version, (u_long)ftch_recexpp->packets,
             (u_long)ftch_recexpp->flows, (u_long)ftch_recexpp->lost,
-            (u_long)ftch_recexpp->reset);
+            (u_long)ftch_recexpp->reset, (u_long)ftch_recexpp->filtered_flows);
 
         }
 
@@ -609,71 +808,24 @@ mcast_done:
 
     } /* stat_inverval */
 
-    /* new client connection ? */ 
-    if (client.num && (FD_ISSET(client.fd, &rfd))) {
-
-      i = -1;
-      invalid = 0;
-
-      /* find an open client slot */
-      for (n = 0; n < client.num; ++n) {
-        if (!(client.recs[n].flags & CLIENT_VALID)) {
-          i = n;
-          break;
-        }
-      }
-
-      /* if > max connections, accept it and exit */
-      if (i == -1) {
-        fterr_warnx("Too many clients");
-        i = CLIENT_MAXCON;
-        invalid = 1;
-      }
-
-      /* accept the connection */
-      len = sizeof(client.recs[i].addr);
-      client.recs[i].fd =
-        accept(client.fd, (struct sockaddr*)&client.recs[i].addr, &len);
-
-      if (client.recs[i].fd < 0)
-        fterr_warn("accept()");
-      else {
-        fterr_info("new client i=%d IP=%s", i,
-          inet_ntoa(client.recs[i].addr.sin_addr));
-        client.recs[i].flags = CLIENT_VALID;
-        client.recs[i].con_time = (u_int32)now;
-      }
-
-#ifdef LIBWRAP
-      request_init(&client.tcpd, RQ_DAEMON, "flow-capture", RQ_FILE,
-        client.recs[i].fd, NULL);
-
-      fromhost(&client.tcpd);
-
-      if (!hosts_access(&client.tcpd)) {
-        fterr_warnx("refused by libwrap");
-        close(client.recs[i].fd);
-        client.recs[i].flags = 0;
-        invalid = 0;
-      }
-#endif /* LIBWRAP */
-
-      if (invalid) {
-        close(client.recs[i].fd);
-        client.recs[i].flags = 0;
-      }
-
-    } /* new client */
-
     /* flag for work later on */
     ftpdu.ftd.count = 0;
    
     /* PDU ready */
     if (FD_ISSET(ftnet.fd, &rfd)) {
 
-      if ((ftpdu.bused = recvmsg(ftnet.fd, &ftnet.msg, 0)) == -1)
+restart_recvmsg:
+
+      if ((ftpdu.bused = recvmsg(ftnet.fd,
+        (struct msghdr*)&ftnet.msg, 0)) < 0) {
+
+        if (errno == EAGAIN)
+          goto restart_recvmsg;
+
         fterr_err(1, "recvmsg()");
 
+      }
+
 #ifdef IP_RECVDSTADDR
       /* got destination IP back? */
       if ((ftnet.msgip.hdr.cmsg_level == IPPROTO_IP) &&
@@ -770,7 +922,7 @@ mcast_done:
         fmt_ipv4(fmt_src_ip, ftch_recexp.src_ip, FMT_JUST_LEFT);
         fmt_ipv4(fmt_dst_ip, ftch_recexp.dst_ip, FMT_JUST_LEFT);
         fterr_info("New exporter: time=%lu src_ip=%s dst_ip=%s d_version=%d",
-          (u_long)now, fmt_src_ip, fmt_dst_ip, (int)ftpdu.ftv.d_version);
+          (u_long)tt_now, fmt_src_ip, fmt_dst_ip, (int)ftpdu.ftv.d_version);
 
         /* set translation function */
         if (ftch_recexp.d_version != ftv.d_version)
@@ -803,7 +955,7 @@ mcast_done:
       /* decode the pdu */
       ftpdu.ftd.byte_order = ftset.byte_order;
       ftpdu.ftd.as_sub = ftset.as_sub;
-      ftpdu.ftd.exporter_ip = ftnet.rem_ip;
+      ftpdu.ftd.exporter_ip = ftch_recexp.src_ip;
       n = fts3rec_pdu_decode(&ftpdu);
 
       /* update the exporter stats */
@@ -822,7 +974,7 @@ skip_pdu_decode:
         fterr_errx(1, "calc_rotate(): failed");
 
       /* remember when file was created */
-      cap_file.time = (u_int32)now;
+      cap_file.time = (u_int32)tt_now;
 
       /* remember the version encoded in the filename */
       bcopy(&ftv, &cap_file.ftv, sizeof cap_file.ftv);
@@ -849,13 +1001,15 @@ skip_pdu_decode:
       if (ftio_set_ver(&ftio, &ftv) < 0)
         fterr_errx(1, "ftio_set_ver(): failed");
 
+      /* need offsets for filter later */
+      fts3rec_compute_offsets(&fo, &ftv);
+
       ftio_set_comment(&ftio, ftset.comments);
       ftio_set_cap_hostname(&ftio, ftset.hnbuf);
       ftio_set_byte_order(&ftio, ftset.byte_order);
       ftio_set_z_level(&ftio, ftset.z_level);
       ftio_set_cap_time(&ftio, cap_file.time, 0);
       ftio_set_debug(&ftio, debug);
-      ftio_set_xip(&ftio, ftnet.rem_ip);
       ftio_set_corrupt(&ftio, cap_file.hdr_flows_corrupt);
       ftio_set_lost(&ftio, cap_file.hdr_flows_lost);
       ftio_set_reset(&ftio, cap_file.hdr_flows_reset);
@@ -871,37 +1025,115 @@ skip_pdu_decode:
 
     } /* create capture file and init new io stream */
 
-    /* if the decode buffer has entries write them out */
+    /* load filters and tags? */
+    if (reload_flag && ftv.set) {
 
+      /* load tags */
+      if (tag_active) {
+  
+        /* not first time through, then free previous tags */
+        if (ftd) {
+          fttag_free(&fttag);
+          fterr_info("Reloading tags.");
+        }
+    
+        if (fttag_load(&fttag, tag_fname) < 0)
+          fterr_errx(1, "fttag_load(): failed");
+    
+        if (!(ftd = fttag_def_find(&fttag, tag_active)))
+          fterr_errx(1, "fttag_load(): failed");
+    
+      } /* tag_active */
+    
+      /* load filters */
+      if (filter_active) {
+  
+        /* not first time through, then free previous filters */
+        if (ftfd) {
+          ftfil_free(&ftfil);
+          fterr_info("Reloading filters.");
+        }
+        
+        if (ftfil_load(&ftfil, filter_fname))
+          fterr_errx(1, "ftfil_load(%s): failed", filter_fname);
+            
+        if (!(ftfd = ftfil_def_find(&ftfil, filter_active)))
+          fterr_errx(1, "ftfil_def_find(%s): failed", filter_active);
+
+        if (ftfil_def_test_xfields(ftfd, ftrec_xfield(&ftv)))
+          fterr_errx(1, "Filter references a field not in flow.");
+    
+      } /* filter_active */
+
+      reload_flag = 0;
+
+    } /* reload_flag */
+
+    /* if the decode buffer has entries write them out */
     for (i = 0, offset = 0; i < ftpdu.ftd.count;
       ++i, offset += ftpdu.ftd.rec_size) {
 
+      /* translate version? */
+      if (ftch_recexpp->xlate) {
+ 
+        ftch_recexpp->xlate(ftpdu.ftd.buf+offset, &xl_rec);
+
+        out_rec = (char*)&xl_rec;
+
+        /* tagging? */
+        if (tag_active)
+          fttag_def_eval(ftd, (struct fts3rec_v1005*)out_rec);
+
+      } else {
+
+        out_rec = (char*)ftpdu.ftd.buf+offset;
+
+      }
+
+      /* filter? */
+      if (ftfd)
+        if (ftfil_def_eval(ftfd, out_rec, &fo) == FT_FIL_MODE_DENY) {
+          ++ftch_recexpp->filtered_flows;
+          continue; 
+        }
+
+      /* update # of flows stored in capture file */
+      cap_file.hdr_nflows ++;
+
       /* simple data privacy */
       if (privacy_mask != 0xFFFFFFFF)
-        ftrec_mask_ip(ftpdu.ftd.buf+offset, &ftpdu.ftv, &ftipmask);
-
-        /* translate version? */
-        if (ftch_recexpp->xlate) {
+        ftrec_mask_ip(out_rec, &ftv, &ftipmask);
  
-          ftch_recexpp->xlate(ftpdu.ftd.buf+offset, &out_rec);
- 
-          if ((n = ftio_write(&ftio, &out_rec)) < 0)
-            fterr_errx(1, "ftio_write(): failed");
+      if ((n = ftio_write(&ftio, out_rec)) < 0)
+        fterr_errx(1, "ftio_write(): failed");
 
-        } else {
+      /* write to clients */
+      FT_LIST_FOREACH(client_rec, &client.list, chain) {
 
-          if ((n = ftio_write(&ftio, ftpdu.ftd.buf+offset)) < 0)
-            fterr_errx(1, "ftio_write(): failed");
+        if ((n = ftio_write(&client_rec->ftio, out_rec)) < 0) {
 
-        }
+          fterr_info("Killed client: ip=%s, dtime=%lu",
+            inet_ntoa(client_rec->addr.sin_addr),
+            (unsigned long)tt_now - client_rec->conn_time);
+
+          ftio_close(&client_rec->ftio);
+          client_rec2 = client_rec;
+          client_rec = client_rec->chain.le_next;
+          FT_LIST_REMOVE(client_rec2, chain);
+          free(client_rec2);
+          --client.active;
+
+          if (!client_rec)
+            break;
+
+        } /* ftio_write */
+
+      } /* foreach client */
 
       /* update # of bytes stored in capture file */
       cap_file.nbytes += n;
 
-    }
-
-    /* update # of flows stored in capture file */
-    cap_file.hdr_nflows += ftpdu.ftd.count;
+    } /* foreach entry in decode buffer */
 
     /*
      * time for a new file ?
@@ -915,7 +1147,7 @@ skip_pdu_decode:
             
       if (cap_file.fd != -1) {
 
-        ftio_set_cap_time(&ftio, cap_file.time, (u_int32)now);
+        ftio_set_cap_time(&ftio, cap_file.time, (u_int32)tt_now);
         ftio_set_corrupt(&ftio, cap_file.hdr_flows_corrupt);
         ftio_set_lost(&ftio, cap_file.hdr_flows_lost);
         ftio_set_reset(&ftio, cap_file.hdr_flows_reset);
@@ -939,15 +1171,35 @@ skip_pdu_decode:
           fterr_err(1, "rename(%s,%s)", cap_file.name, cap_file.nname);
 
         /* add it to the ager */
-        if (ftfile_add_tail(&fte, cap_file.nname, cap_file.nbytes,
-          cap_file.time))
-          fterr_errx(1, "ftfile_add_tail(%s): failed", cap_file.name);
+        if (fte.expiring)
+          if (ftfile_add_tail(&fte, cap_file.nname, cap_file.nbytes,
+            cap_file.time))
+            fterr_errx(1, "ftfile_add_tail(%s): failed", cap_file.name);
 
         /* debugging gets a dump of the ager */
         if (debug)
           ftfile_dump(&fte);
 
-        /* reset */
+	/* Do the post rotate exec */
+        if (post_rotate_exec[0]) {
+
+          if ((n = fork()) == -1) {
+
+            fterr_err(1, "fork()");
+
+          } else if (!n) { /* child */
+
+            n = execl(post_rotate_exec, post_rotate_exec, cap_file.nname,
+                NULL);
+
+            if (n == -1) 
+              fterr_err(1, "exec(%s)", post_rotate_exec);
+
+            _exit(0);
+          } /* child */
+        } /* post rotate exec */
+        
+	/* reset */
         bzero(&cap_file, sizeof cap_file);
 
         /* invalidate file descriptor */
@@ -964,10 +1216,44 @@ skip_pdu_decode:
     if (sig_quit_flag)
       goto main_exit;
 
-    if (!(cap_file.hdr_nflows % 1000)) {
+  /*
+   * If client attachments are enabled, and the flow version has been
+   * determined setup a listener, but only once.
+   */
+  if (ftv.set && client.max && !client.enabled) {
+    
+    /* socket for listen */
+    if ((client.fd = socket(AF_INET, SOCK_STREAM, 0)) < 0)
+      fterr_err(1, "socket()");
 
-      if (ftfile_expire(&fte, enable_unlink, cap_file.nbytes))
-        fterr_errx(1, "ftfile_expire(): failed");
+    one = 1;
+
+    if (setsockopt(client.fd, SOL_SOCKET, SO_REUSEADDR, (char*)&one,
+      sizeof (one)))
+      fterr_err(1, "setsockopt(SO_REUSEADDR)");
+      
+    if (bind(client.fd, (struct sockaddr*)&client.addr, sizeof (client.addr))
+      < 0)
+      fterr_err(1, "bind(%s)", inet_ntoa(client.addr.sin_addr));
+      
+    /* listen for new TCP connections */
+    if (listen(client.fd, 5) < 0)
+      fterr_err(1, "listen()");
+
+    /* non blocking */
+    if (fcntl(client.fd, F_SETFL, O_NONBLOCK) < 0)
+      fterr_err(1, "fcntl()");
+
+    client.enabled = 1;
+
+  } /* clients enabled */
+
+
+    if (!(cap_file.hdr_nflows % 1001)) {
+
+      if (fte.expiring)
+        if (ftfile_expire(&fte, enable_unlink, cap_file.nbytes))
+          fterr_errx(1, "ftfile_expire(): failed");
 
     } /* ager run? */
 
@@ -975,87 +1261,74 @@ skip_pdu_decode:
 
 main_exit:
 
-  unlink_pidfile(pid, CAPTURE_PIDFILE, ftnet.dst_port);
+  if (pidfile)
+    unlink_pidfile(pid, pidfile, ftnet.dst_port);
 
   if (sig_quit_flag)
     fterr_info("SIGQUIT");
 
   /* free storage allocated to file entries */
-  ftfile_free(&fte);
+  if (fte.expiring)
+    ftfile_free(&fte);
 
   return 0;
 
 } /* main */
 
-void usage() {
-
-  fprintf(stderr, "flow-capture: [options] locip/remip/port\n");
-  fprintf(stderr, " -A #  replace exports with AS 0 with #\n");
-  fprintf(stderr, " -b s  byte order (little or big)\n");
-  fprintf(stderr, " -C s  comment field\n");
-  fprintf(stderr, " -c #  enable n flow clients\n");
-  fprintf(stderr, " -d #  debug level.\n");
-  fprintf(stderr, " -D    do not daemonize.\n");
-  fprintf(stderr, " -e #  files to keep before expire. (0=disable)\n");
-  fprintf(stderr, " -E #  bytes to keep before expire. (0=disable)\n");
-  fprintf(stderr, " -m m  privacy mask, ie 255.255.255.0\n");
-  fprintf(stderr, " -n #  file rotations per day.\n");
-  fprintf(stderr, " -N #  directory nesting level.\n");
-  fprintf(stderr, " -s #  size of files to keep before expire. (0=disable)\n"
-);
-  fprintf(stderr, "       use b,K,M,G as multipliers, ie 16M for 16*1024*1024\n");
-  fprintf(stderr, " -S #  receiver stat interval in minutes\n");
-  fprintf(stderr, " -V #  version of export to capture.  Use #.# for V8\n");
-  fprintf(stderr, "   1   - no sequence #, engine id, or AS\n");
-  fprintf(stderr, "   5   - 1+ sequence #, engine id, and AS\n");
-  fprintf(stderr, "   6   - 5+ encapsulation size\n");
-  fprintf(stderr, "   7   - 5+ router shortcut (Catalyst switches)\n");
-  fprintf(stderr, "   8.1 - AS aggregation\n");
-  fprintf(stderr, "   8.2 - Protocol Port aggregation\n");
-  fprintf(stderr, "   8.3 - Source Prefix aggregation\n");
-  fprintf(stderr, "   8.4 - Destination Prefix aggregation\n");
-  fprintf(stderr, "   8.5 - Prefix aggregation\n");
-  fprintf(stderr, " -w x  working directory pathname.\n");
-  fprintf(stderr, " signals\n");
-  fprintf(stderr, "   SIGHUP  - close and rotate current file\n");
-  fprintf(stderr, "   SIGQUIT - close current file and exit\n");
-  fprintf(stderr, "\n");
-    
-} /* usage */ 
-
-void sig_pipe()
+void sig_pipe(int signo)
 {
   sig_pipe_flag = 1;
 }
     
-void sig_hup()
+void sig_hup(int signo)
 {
   sig_hup_flag = 1;
+  reload_flag = 1;
 }
     
-void sig_quit()
+void sig_quit(int signo)
 {  
   sig_quit_flag = 1;
 }
 
-int calc_rotate (int next, u_int32 *trotate, int *cur)
-{
+void sig_chld(int signo)
+{  
+  sig_chld_flag = 1;
+}
 
-  time_t now;       /* current time */
-  u_long irotate;   /* interval of next in seconds */
+/* doubletime - like time(2), but returns a double (with fractional seconds)
+ * This was inspired by the Time::HiRes perl module.  E.g.:
+ *    $ perl -MTime::HiRes -le 'print scalar Time::HiRes::time'
+ * See "perl/CPAN/authors/id/D/DE/DEWEG/Time-HiRes-01.20.tar.gz".
+ * - Dave Plonka <plonka@doit.wisc.edu>
+ */
+double doubletime(void) {
+   double now;
+   struct timeval tv_now;
+   if (-1 == gettimeofday(&tv_now, (struct timezone *)0))
+      return -1;
+   now = tv_now.tv_sec + (tv_now.tv_usec / 1000000.0);
+   return now;
+}
+
+int calc_rotate (int next, double *trotate, int *cur)
+{
+  double now;       /* current time */
+  time_t tt_now;    /* current time */
+  double irotate;   /* interval of next in seconds */
   time_t morning;   /* start of the day */
   struct tm *tm1;
         
-  irotate = 86400 / (next+1);
+  irotate = 86400. / (next+1);
        
   *cur = 0;
   
-  if ((now = time((time_t)0L)) == -1)
+  if (-1 == (tt_now = now = doubletime()))
     return -1;
-    
+   
   /* calculate start of the day */
-  if (!(tm1 = localtime (&now)))
-    return -1;
+  if (!(tm1 = localtime (&tt_now)))
+     return -1;
 
   tm1->tm_sec = 0;
   tm1->tm_min = 0;
@@ -1067,7 +1340,7 @@ int calc_rotate (int next, u_int32 *trotate, int *cur)
   if (next)
     *trotate = morning + irotate;
   else  
-    *trotate = morning + 86400;
+    *trotate = morning + 86400.;
   
   while (now > *trotate) {
   
@@ -1082,9 +1355,23 @@ int calc_rotate (int next, u_int32 *trotate, int *cur)
 
 void fterr_exit_handler(int code)
 {
-  if (pid)
-    unlink_pidfile(pid, CAPTURE_PIDFILE, ftnet.dst_port);
+  if (pid && pidfile)
+    unlink_pidfile(pid, pidfile, ftnet.dst_port);
   exit (code);
 } /* fterr_exit_handler */
 
+void usage(void) {
+
+  fprintf(stderr, "Usage: flow-capture [-h] [-A AS0_substitution] [-b big|little]\n");
+  fprintf(stderr, "       [-C comment] [-c flow_clients] [-d debug_level] [-D daemonize]\n");
+  fprintf(stderr, "       [-e expire_count] [-E expire_size[bKMG]] [-m privacy_mask]\n");
+  fprintf(stderr, "       [-n rotations] [-N nesting_level] [-p pidfile ] [-R rotate_program]\n");
+  fprintf(stderr, "       [-S stat_interval] [-t tag_fname] [-T tag_active] [-V pdu_version]\n");
+  fprintf(stderr, "       [-z z_level] -w workdir localip/remoteip/port\n");
+  fprintf(stderr, "Signals:\n");
+  fprintf(stderr, "   SIGHUP  - close and rotate current file\n");
+  fprintf(stderr, "   SIGQUIT - close current file and exit\n");
+fprintf(stderr, "\n%s version %s: built by %s\n", PACKAGE, VERSION, FT_PROG_BUILD);
+
+} /* usage */ 
 
