@@ -23,12 +23,11 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- *      $Id: flow-dscan.c,v 1.17 2001/04/08 12:41:20 maf Exp $
+ *      $Id: flow-dscan.c,v 1.24 2003/02/13 02:38:59 maf Exp $
  */
 
-#if HAVE_CONFIG_H
- #include <config.h>
-#endif
+#include "ftconfig.h"
+#include "ftlib.h"
 
 #include <sys/time.h>
 #include <sys/types.h>
@@ -37,21 +36,21 @@
 #include <unistd.h>
 #include <stdio.h>
 #include <stdlib.h>
-#if HAVE_STRINGS_H
- #include <strings.h>
-#endif
-#if HAVE_STRING_H
-  #include <string.h>
-#endif
 #include <time.h>
 #include <fcntl.h>
 #include <signal.h>
 #include <syslog.h>
-#include "ftlib.h"
-#include "fmt.h"
+
+#if HAVE_STRINGS_H
+ #include <strings.h>
+#endif
+
+#if HAVE_STRING_H
+  #include <string.h>
+#endif
+
+#include "ftbuild.h"
 #include "flow-dscan.h"
-#include "support.h"
-#include "bit1024.h"
 
 /*
  * Detect network scan's
@@ -80,7 +79,6 @@
 
 int debug;
 int sig_hup, sig_usr1;
-char *progname = "flow-dscan\0";
 
 int load_state(struct dscan_state *ds);
 void flow_dump(struct fts3rec_gen *rec);
@@ -95,8 +93,6 @@ int dump_state(struct dscan_state *ds);
 
 int main(int argc, char **argv)
 {
-  extern char *optarg;
-  extern int optind, debug;
   struct ftio ftio;
   struct ftprof ftp;
   struct fts3rec_gen *rec;
@@ -135,7 +131,7 @@ int main(int argc, char **argv)
   /* init fterr */
   fterr_setid(argv[0]);
 
-  while ((i = getopt(argc, argv, "d:i:I:Dh?lmps:t:wL:O:P:S:T:bBW")) != -1)
+  while ((i = getopt(argc, argv, "d:i:I:D:h?lmps:t:wL:O:P:S:T:bBW")) != -1)
 
     switch (i) {
 
@@ -227,6 +223,9 @@ int main(int argc, char **argv)
       break;
     } /* switch */
 
+  if (argc - optind)
+    fterr_errx(1, "Extra arguments starting with %s.", argv[optind]);
+
   /* daemonize */
   if (!no_detach) {
     if ((pid = fork()) == -1) {
@@ -259,8 +258,11 @@ int main(int argc, char **argv)
   ftprof_start (&ftp);
 
   /* configure handlers */
-  signal(SIGHUP, sig_hup_handler);
-  signal(SIGUSR1, sig_usr1_handler);
+  if (mysignal(SIGHUP, sig_hup_handler) == SIG_ERR)
+    fterr_err(1, "signal(SIGHUP)");
+
+  if (mysignal(SIGUSR1, sig_usr1_handler) == SIG_ERR)
+    fterr_err(1, "signal(SIGUSR1)");
 
   /* initialize scanner hash table */
   for (i = 0; i < DSCAN_HASHSIZE; ++i) {
@@ -703,49 +705,18 @@ int load_state(struct dscan_state *ds)
   return 0;
 } /* load_state */
 
-void usage(void) {
-
-  fprintf(stderr, "flow-dscan:\n\n");
-  fprintf(stderr, " -d #  set debug level.\n");
-  fprintf(stderr, " -b do not detach\n");
-  fprintf(stderr, " -i [!](list) input interface list filter\n");
-  fprintf(stderr, " -I [!](list) output interface list filter\n");
-  fprintf(stderr, " -l load state\n");
-  fprintf(stderr, " -m enable multicast address filter\n");
-  fprintf(stderr, " -p dump state on exit\n");
-  fprintf(stderr, " -s state file\n");
-  fprintf(stderr, " -t ager timeout (0 to disable ager)\n");
-  fprintf(stderr, " -w filter inbound www (prot=6, srcPort=80, dstPort>1023)\n");
-  fprintf(stderr, " -B do not detach, leave syslog enabled\n");
-  fprintf(stderr, " -D depth of dst ip list (max 255)\n");
-  fprintf(stderr, " -L suppress list file\n");
-  fprintf(stderr, " -S trigger for port scan (max 1024)\n");
-  fprintf(stderr, " -O # trigger warning on excessive flow octets\n");
-  fprintf(stderr, " -P # trigger warning on excessive flow packets\n");
-  fprintf(stderr, " -T # trigger warning on excessive flow time (duration)\n");
-  fprintf(stderr, " -W filter outbound www (prot=6, dstPort=80, srcPort>1023)\n");
-  fprintf(stderr, " signals\n");
-  fprintf(stderr, "   SIGHUP  - reload suppress list\n");
-  fprintf(stderr, "   SIGUSR1 - dump state\n");
-
-  fprintf(stderr, "\n\n");
-} /* usage */
-
 void sig_usr1_handler(int sig)
 {
-  extern int sig_usr1;
   sig_usr1 = 1;
 } /* sig_usr1_handler */
 
 void sig_hup_handler(int sig)
 {
-  extern int sig_hup;
   sig_hup = 1;
 } /* sig_usr1_handler */
 
 void ager(struct dscan_state *ds, u_int32 total_flows32)
 {
-
   static int ager_i;
   int i, work;
   struct dscan_rec *drp, *odrp, *drp2;
@@ -1062,3 +1033,16 @@ void flow_dump(struct fts3rec_gen *rec)
 
 } /* flow_dump */
 
+void usage(void) {
+
+  fprintf(stderr, "Usage: flow-dscan [-bBhlmpwW] [-d debug_level] [-D iplist_depth]\n");
+  fprintf(stderr, "       [-s state_file] [-i input_filter] [-L suppress_list]\n");
+  fprintf(stderr, "       [-o output_filter] [-O excessive_octets] [-P excessive_flows]\n");
+  fprintf(stderr, "       [-S port_scan_trigger]  [-t ager_timeout]\n");
+  fprintf(stderr, "Signals:\n");
+  fprintf(stderr, "   SIGHUP  - reload suppress list\n");
+  fprintf(stderr, "   SIGUSR1 - dump state\n");
+  fprintf(stderr, "\n%s version %s: built by %s\n", PACKAGE, VERSION, FT_PROG_BUILD);
+
+} /* usage */
+
