@@ -23,51 +23,47 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- *      $Id: flow-receive.c,v 1.31 2001/07/17 04:13:37 maf Exp $
+ *      $Id: flow-receive.c,v 1.50 2003/02/24 02:55:15 maf Exp $
  */
 
-#if HAVE_CONFIG_H
- #include <config.h>
-#endif
-
+#include "ftconfig.h"
+#include "ftlib.h"
 
 #include <sys/time.h>
 #include <sys/types.h>
 #include <sys/uio.h>
-#include <sys/errno.h>
 #include <sys/socket.h>
 #include <sys/stat.h>
-#include <signal.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
+#include <errno.h>
 #include <unistd.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <signal.h>
+#include <time.h>
+#include <fcntl.h>
+
 #if HAVE_STRINGS_H
  #include <strings.h>
 #endif
+
 #if HAVE_STRING_H
   #include <string.h>
 #endif
-#include <time.h>
-#include <fcntl.h>
-#include <support.h>
-#include <ftlib.h>
-#include <fmt.h>
+
+#include "ftbuild.h"
 
 int debug;
 int done;
 
-void usage();
-void sig_quit();
+void usage(void);
+void sig_quit(int);
 
 #define SELECT_TIMEOUT 1   /* 1 second */
 
 int main(int argc, char **argv)
 {
-  extern char *optarg;
-  extern int optind, errno;
-  extern int done;
 #ifdef IP_ADD_MEMBERSHIP
   struct sockaddr_in tmp_addr;
   struct ip_mreq mr;
@@ -87,21 +83,30 @@ int main(int argc, char **argv)
   struct ftipmask ftipmask;
   struct ftchash *ftch;
   struct ftchash_rec_exp ftch_recexp, *ftch_recexpp;
-  time_t now;
+  struct fttag fttag;
+  struct ftfil ftfil;
+  struct fttag_def *ftd;
+  struct ftfil_def *ftfd;
+  struct fts3rec_offsets fo;
+  time_t now, time_startup;
   int i, n, offset, out_fd, out_fd_plain, one;
   unsigned int v1, v2;
   fd_set rfd;
   char *out_fname;
-  u_int32 nflows, time_start, time_end;
+  u_int32 filtered_flows, nflows, time_start, time_end;
   u_int32 flows_corrupt, flows_lost, flows_reset;
   u_int32 privacy_mask, hash;
   char fmt_src_ip[32], fmt_dst_ip[32], fmt_dst_port[32];
-  char out_rec[FT_IO_MAXREC];
+  char xl_rec[FT_IO_MAXREC], *out_rec;
+  char *tag_fname, *tag_active;
+  char *filter_fname, *filter_active;
   int stat_interval, stat_next;
 
+  time_startup = time((time_t)0L);
+
   privacy_mask = 0xFFFFFFFF;
   out_fname = (char*)0L;
-  nflows = 0;
+  filtered_flows = nflows = 0;
   out_fd_plain = 0;
   out_fd = -1;
   ftset.z_level = 0;
@@ -111,10 +116,21 @@ int main(int argc, char **argv)
   bzero(&ftv, sizeof ftv);
   bzero(&ftpdu, sizeof ftpdu);
   bzero(&ftch_recexp, sizeof ftch_recexp);
+  bzero(&fttag, sizeof fttag);
+  bzero(&ftfil, sizeof ftfil);
   flows_corrupt = flows_lost = flows_reset = 0;
   stat_interval = 0;
   stat_next = -1;
 
+  tag_fname = FT_PATH_CFG_TAG;
+  tag_active = (char*)0L;
+
+  filter_fname = FT_PATH_CFG_FILTER;
+  filter_active = (char*)0L;
+
+  ftfd = (struct ftfil_def*)0L;
+  ftd = (struct fttag_def*)0L;
+
   /* init fterr */
   fterr_setid(argv[0]);
 
@@ -126,7 +142,7 @@ int main(int argc, char **argv)
   ftnet.loc_addr.sin_addr.s_addr = htonl(INADDR_ANY);
   ftnet.loc_addr.sin_port = htons(FT_PORT);
 
-  while ((i = getopt(argc, argv, "A:b:C:d:h?m:o:S:V:z:")) != -1)
+  while ((i = getopt(argc, argv, "A:b:C:d:f:F:h?m:o:S:t:T:V:z:")) != -1)
 
     switch (i) {
 
@@ -151,6 +167,14 @@ int main(int argc, char **argv)
       debug = atoi(optarg);
       break;
 
+    case 'f': /* filter fname */
+      filter_fname = optarg;
+      break;
+
+    case 'F': /* filter active */
+      filter_active = optarg;
+      break;
+
     case 'h': /* help */
     case '?':
       usage();
@@ -171,6 +195,14 @@ int main(int argc, char **argv)
         fterr_errx(1, "Stat interval must be between 0 and 60.");
       break;
 
+    case 't': /* tag filename */
+      tag_fname = optarg;
+      break;
+      
+    case 'T': /* active tag */
+      tag_active = optarg;
+      break;
+
     case 'V': /* PDU version */
       n = sscanf(optarg, "%u.%u", &v1, &v2);
       if (n == 1) {
@@ -201,6 +233,36 @@ int main(int argc, char **argv)
 
     } /* switch */
 
+  /* load tags */
+  if (tag_active) {
+
+    if (ftv.set)
+      fterr_errx(1, "-V not supported with tagging.");
+
+    if (fttag_load(&fttag, tag_fname) < 0)
+      fterr_errx(1, "fttag_load(%s): failed", tag_fname);
+
+    if (!(ftd = fttag_def_find(&fttag, tag_active))) 
+      fterr_errx(1, "fttag_def_find(%s): failed", tag_active);
+
+    /* required for fttag_eval() */
+    ftv.s_version = FT_IO_SVERSION;
+    ftv.d_version = 1005;
+    ftv.set = 1;
+
+  } /* tag_active */
+
+  /* load filters */
+  if (filter_active) {
+
+    if (ftfil_load(&ftfil, filter_fname))
+      fterr_errx(1, "ftfil_load(%s): failed", filter_fname);
+
+    if (!(ftfd = ftfil_def_find(&ftfil, filter_active)))
+      fterr_errx(1, "ftfil_def_find(%s): failed", filter_active);
+
+  } /* filter_active */
+
   /* setup for ftrec_mask_ip */
   if (privacy_mask != 0xFFFFFFFF)
     ftrec_compute_mask(&ftipmask, privacy_mask, privacy_mask,
@@ -235,17 +297,17 @@ int main(int argc, char **argv)
    * configure signal handlers
    */
 
-  if (signal(SIGQUIT, sig_quit) < 0)
-    fterr_err(1, "signal(");
+  if (mysignal(SIGQUIT, sig_quit) == SIG_ERR)
+    fterr_err(1, "signal(SIGQUIT)");
 
-  if (signal(SIGHUP, sig_quit) < 0)
-    fterr_err(1, "signal(");
+  if (mysignal(SIGHUP, sig_quit) == SIG_ERR)
+    fterr_err(1, "signal(SIGHUP)");
 
-  if (signal(SIGINT, sig_quit) < 0)
-    fterr_err(1, "signal(");
+  if (mysignal(SIGINT, sig_quit) == SIG_ERR)
+    fterr_err(1, "signal(SIGINT)");
 
-  if (signal(SIGTERM, sig_quit) < 0)
-    fterr_err(1, "signal(");
+  if (mysignal(SIGTERM, sig_quit) == SIG_ERR)
+    fterr_err(1, "signal(SIGTERM)");
 
   /* get hostname */
   if (gethostname((char*)&ftset.hnbuf, (int)FT_HOSTNAME_LEN-1) == -1)
@@ -256,7 +318,7 @@ int main(int argc, char **argv)
   if ((ftnet.fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
     fterr_err(1, "socket()");
 
-  if (bigsockbuf(ftnet.fd, SO_RCVBUF, 224*1024) < 0)
+  if (bigsockbuf(ftnet.fd, SO_RCVBUF, FT_SO_RCV_BUFSIZE) < 0)
     fterr_err(1, "bigsockbuf()");
 
 /* multicast capable? */
@@ -379,7 +441,10 @@ mcast_done:
   ftio_set_cap_hostname(&ftio, ftset.hnbuf);
   ftio_set_byte_order(&ftio, ftset.byte_order);
   ftio_set_z_level(&ftio, ftset.z_level);
-  ftio_set_cap_time(&ftio, time_start, 0);
+  if (out_fd_plain)
+    ftio_set_cap_time(&ftio, time_start, 0);
+  else
+    ftio_set_cap_time_start(&ftio, time_start);
   ftio_set_debug(&ftio, debug);
   ftio_set_streaming(&ftio, 1);
 
@@ -391,7 +456,6 @@ mcast_done:
     ftio_set_corrupt(&ftio, flows_corrupt);
     ftio_set_lost(&ftio, flows_lost);
     ftio_set_reset(&ftio, flows_reset);
-    ftio_set_xip(&ftio, ftnet.rem_ip);
     ftio_set_flows_count(&ftio, nflows);
   }
 
@@ -401,6 +465,9 @@ mcast_done:
     if (ftio_set_ver(&ftio, &ftv) < 0)
       fterr_errx(1, "ftio_set_ver(): failed");
 
+    /* need offsets for filter later */
+    fts3rec_compute_offsets(&fo, &ftv);
+
     /* header first */
     if (ftio_write_header(&ftio) < 0)
       fterr_errx(1, "ftio_write_header()");
@@ -464,8 +531,9 @@ mcast_done:
           fmt_ipv4(fmt_dst_ip, ftch_recexpp->dst_ip, FMT_JUST_LEFT);
 
           fterr_info(
-            "STAT: now=%lu src_ip=%s dst_ip=%s d_ver=%d pkts=%lu flows=%lu lost=%lu reset=%lu",
-            (unsigned long)now, fmt_src_ip, fmt_dst_ip,
+            "STAT: now=%lu startup=%lu src_ip=%s dst_ip=%s d_ver=%d pkts=%lu flows=%lu lost=%lu reset=%lu",
+            (unsigned long)now, (unsigned long)time_startup,
+            fmt_src_ip, fmt_dst_ip,
             ftch_recexpp->d_version, (u_long)ftch_recexpp->packets,
             (u_long)ftch_recexpp->flows, (u_long)ftch_recexpp->lost,
             (u_long)ftch_recexpp->reset);
@@ -486,9 +554,18 @@ mcast_done:
 
     if (FD_ISSET(ftnet.fd, &rfd)) {
 
-      if ((ftpdu.bused = recvmsg(ftnet.fd, &ftnet.msg, 0)) == -1)
+restart_recvmsg:
+
+      if ((ftpdu.bused = recvmsg(ftnet.fd,
+        (struct msghdr*)&ftnet.msg, 0)) < 0) {
+
+        if (errno == EAGAIN)
+          goto restart_recvmsg;
+
         fterr_err(1, "recvmsg()");
 
+      }
+
 #ifdef IP_RECVDSTADDR
       /* got destination IP back? */
       if ((ftnet.msgip.hdr.cmsg_level == IPPROTO_IP) &&
@@ -547,6 +624,9 @@ mcast_done:
         /* flag struct as configured */
         ftv.set = 1;
 
+        /* need offsets for filter later */
+        fts3rec_compute_offsets(&fo, &ftv);
+
         /* header first */
         if (ftio_write_header(&ftio) < 0)
           fterr_errx(1, "ftio_write_header(): failed");
@@ -627,7 +707,7 @@ mcast_done:
       /* decode */
       ftpdu.ftd.byte_order = ftset.byte_order;
       ftpdu.ftd.as_sub = ftset.as_sub;
-      ftpdu.ftd.exporter_ip = ftnet.rem_ip;
+      ftpdu.ftd.exporter_ip = ftch_recexp.src_ip;  
       n = fts3rec_pdu_decode(&ftpdu);
 
       /* update the exporter stats */
@@ -637,26 +717,38 @@ mcast_done:
       /* write decoded flows */
       for (i = 0, offset = 0; i < n; ++i, offset += ftpdu.ftd.rec_size) {
 
+        /* translate version? */
+        if (ftch_recexpp->xlate) {
+
+          ftch_recexpp->xlate(ftpdu.ftd.buf+offset, &xl_rec);
+
+          out_rec = (char*)&xl_rec;
+
+          /* tagging? */
+          if (tag_active)
+            fttag_def_eval(ftd, (struct fts3rec_v1005*)out_rec);
+
+        } else {
+
+          out_rec = (char*)ftpdu.ftd.buf+offset;
+
+        }
+
+        /* filter? */
+        if (ftfd)
+          if (ftfil_def_eval(ftfd, out_rec, &fo) == FT_FIL_MODE_DENY) {
+            ++filtered_flows;
+            continue;
+          }
+
         ++nflows;
 
         /* simple data privacy */ 
         if (privacy_mask != 0xFFFFFFFF)
-          ftrec_mask_ip(ftpdu.ftd.buf+offset, &ftpdu.ftv, &ftipmask);
+          ftrec_mask_ip(out_rec, &ftv, &ftipmask);
 
-        /* translate version? */
-        if (ftch_recexpp->xlate) {
-
-          ftch_recexpp->xlate(ftpdu.ftd.buf+offset, &out_rec);
-
-          if (ftio_write(&ftio, &out_rec) < 0)
-            fterr_errx(1, "ftio_write(): failed");
-
-        } else {
-
-          if (ftio_write(&ftio, ftpdu.ftd.buf+offset) < 0)
-            fterr_errx(1, "ftio_write(): failed");
-
-        }
+        if (ftio_write(&ftio, out_rec) < 0)
+          fterr_errx(1, "ftio_write(): failed");
 
       } /* for */
 
@@ -666,14 +758,14 @@ skip1:
 
   } /* while 1 */
 
-  fterr_info("flows processed %lu", (u_long)nflows);
+  fterr_info("flows stored/dropped by filter %lu/%lu", (u_long)nflows,
+    filtered_flows);
 
   /* rewrite header with updated info */
   if (out_fd_plain) {
   
     time_end = (u_int32)time((time_t)0L);
 
-    ftio_set_xip(&ftio, ftnet.rem_ip);
     ftio_set_cap_time(&ftio, time_start, time_end);
     ftio_set_flows_count(&ftio, nflows);
     ftio_set_streaming(&ftio, 0);
@@ -697,34 +789,18 @@ skip1:
 
 } /* main */
 
-void usage() {
-
-  fprintf(stderr, "flow-receive: [options] locip/remip/port\n");
-  fprintf(stderr, " -A #  replace exports with AS 0 with #\n");
-  fprintf(stderr, " -b s  set output byte order (little or big)\n");
-  fprintf(stderr, " -C s  set comment field in output\n");
-  fprintf(stderr, " -d #  set debug level.\n");
-  fprintf(stderr, " -m m  privacy mask, ie 255.255.255.0\n");
-  fprintf(stderr, " -o s  output to file s instead of stdout\n");
-  fprintf(stderr, " -S #  receiver stat interval in minutes\n");
-  fprintf(stderr, " -V #  version of export to capture.  Use #.# for V8\n");
-  fprintf(stderr, "   1   - no sequence #, engine id, or AS\n");
-  fprintf(stderr, "   5   - 1+ sequence #, engine id, and AS\n");
-  fprintf(stderr, "   6   - 5+ encapsulation size\n");
-  fprintf(stderr, "   7   - 5+ router shortcut (Catalyst switches)\n");
-  fprintf(stderr, "   8.1 - AS aggregation\n");
-  fprintf(stderr, "   8.2 - Protocol Port aggregation\n");
-  fprintf(stderr, "   8.3 - Source Prefix aggregation\n");
-  fprintf(stderr, "   8.4 - Destination Prefix aggregation\n");
-  fprintf(stderr, "   8.5 - Prefix aggregation\n");
-  fprintf(stderr, "\n\n");
-
-} /* usage */
-
-void sig_quit(sig)
-int sig;
+void sig_quit(int sig)
 {
-  extern int done;
   done = 1;
 } /* sig_quit */
 
+void usage(void) {
+
+  fprintf(stderr, "Usage: flow-receive [-h] [-A AS0_substitution] [-b big|little] [-C comment]\n");
+  fprintf(stderr, "       [-d debug_level] [-m privacy_mask] [-o output_file] [-S stat_interval]\n");
+  fprintf(stderr, "       [-t tag_fname] [-T tag_active] [-V pdu_version] [-z z_level]\n");
+  fprintf(stderr, "       localip/remoteip/port\n");
+  fprintf(stderr, "\n%s version %s: built by %s\n", PACKAGE, VERSION, FT_PROG_BUILD);
+
+} /* usage */
+
